@@ -1,17 +1,17 @@
 <?php
 /**
- * $Horde: horde/admin/sqlshell.php,v 1.18.10.10 2007/01/02 13:54:03 jan Exp $
+ * $Horde: horde/admin/sqlshell.php,v 1.18.10.14 2008/07/28 17:13:44 chuck Exp $
  *
- * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author Chuck Hagenbuch <chuck@horde.org>
  */
 
 @define('HORDE_BASE', dirname(__FILE__) . '/..');
 require_once HORDE_BASE . '/lib/base.php';
-require_once 'Horde/Menu.php';
-require_once 'Horde/Help.php';
 require_once 'DB.php';
 
 if (!Auth::isAdmin()) {
@@ -21,10 +21,11 @@ if (!Auth::isAdmin()) {
 $title = _("SQL Shell");
 Horde::addScriptFile('stripe.js', 'horde', true);
 require HORDE_TEMPLATES . '/common-header.inc';
-require HORDE_TEMPLATES . '/admin/common-header.inc';
+require HORDE_TEMPLATES . '/admin/menu.inc';
 
 ?>
 <div style="padding:10px">
+<h1 class="header"><?php echo _("SQL Shell") ?></h1><br />
 <form name="sqlshell" action="<?php echo $_SERVER['PHP_SELF'] ?>" method="post">
 <?php Util::pformInput() ?>
 
@@ -39,9 +40,11 @@ $dbh->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS)
 if (Util::getFormData('list-tables')) {
     $description = 'LIST TABLES';
     $result = $dbh->getListOf('tables');
+    $command = null;
 } elseif (Util::getFormData('list-dbs')) {
     $description = 'LIST DATABASES';
     $result = $dbh->getListOf('databases');
+    $command = null;
 } elseif ($command = trim(Util::getFormData('sql'))) {
     // Keep a cache of prior queries for convenience.
     if (!isset($_SESSION['_sql_query_cache'])) {
@@ -60,7 +63,7 @@ if (Util::getFormData('list-tables')) {
 }
 
 if (isset($result)) {
-    if (isset($command)) {
+    if ($command) {
         echo '<h1 class="header">' . _("Query") . '</h1><br /><pre class="text">' . htmlspecialchars($command) . '</pre>';
     }
 
@@ -77,14 +80,14 @@ if (isset($result)) {
                 if ($first) {
                     echo '<tr>';
                     foreach ($row as $key => $val) {
-                        echo '<th align="left">' . (empty($key) ? '&nbsp;' : htmlspecialchars(String::convertCharset($key, $conf['sql']['charset']))) . '</th>';
+                        echo '<th align="left">' . (!strlen($key) ? '&nbsp;' : htmlspecialchars(String::convertCharset($key, $conf['sql']['charset']))) . '</th>';
                     }
                     echo '</tr>';
                     $first = false;
                 }
                 echo '<tr>';
                 foreach ($row as $val) {
-                    echo '<td class="fixed">' . (empty($val) ? '&nbsp;' : htmlspecialchars(String::convertCharset($val, $conf['sql']['charset']))) . '</td>';
+                    echo '<td class="fixed">' . (!strlen($val) ? '&nbsp;' : htmlspecialchars(String::convertCharset($val, $conf['sql']['charset']))) . '</td>';
                 }
                 echo '</tr>';
             }
@@ -98,7 +101,7 @@ if (isset($result)) {
                     echo '<tr><th align="left">' . (isset($description) ? htmlspecialchars($description) : '&nbsp;') . '</th></tr>';
                     $first = false;
                 }
-                echo '<tr><td class="fixed">' . (empty($val) ? '&nbsp;' : htmlspecialchars(String::convertCharset($val, $conf['sql']['charset']))) . '</td></tr>';
+                echo '<tr><td class="fixed">' . (!strlen($val) ? '&nbsp;' : htmlspecialchars(String::convertCharset($val, $conf['sql']['charset']))) . '</td></tr>';
             }
             echo '</table>';
         } else {
@@ -112,7 +115,8 @@ if (isset($result)) {
 
 <?php if (isset($_SESSION['_sql_query_cache']) &&
           count($_SESSION['_sql_query_cache'])): ?>
-  <select name="query_cache" onchange="document.sqlshell.sql.value = document.sqlshell.query_cache[document.sqlshell.query_cache.selectedIndex].value;">
+  <label for="query_cache" class="hidden"><?php echo ("Query cache") ?></label>
+  <select id="query_cache" name="query_cache" onchange="document.sqlshell.sql.value = document.sqlshell.query_cache[document.sqlshell.query_cache.selectedIndex].value;">
   <?php foreach ($_SESSION['_sql_query_cache'] as $query): ?>
     <option value="<?php echo htmlspecialchars($query) ?>"><?php echo htmlspecialchars($query) ?></option>
   <?php endforeach; ?>
@@ -122,12 +126,13 @@ if (isset($result)) {
   <br />
 <?php endif; ?>
 
-<textarea class="fixed" name="sql" rows="10" cols="60">
-<?php if (!empty($command)) echo htmlspecialchars($command) ?></textarea>
+<label for="sql" class="hidden"><?php echo ("SQL Query") ?></label>
+<textarea class="fixed" id="sql" name="sql" rows="10" cols="60">
+<?php if (strlen($command)) echo htmlspecialchars($command) ?></textarea>
 <br />
 <input type="submit" class="button" value="<?php echo _("Execute") ?>" />
 <input type="button" class="button" value="<?php echo _("Clear Query") ?>" onclick="document.sqlshell.sql.value=''" />
-<?php if (!empty($command)): ?>
+<?php if (strlen($command)): ?>
 <input type="reset" class="button" value="<?php echo _("Restore Last Query") ?>" />
 <?php endif; ?>
 <?php if ($dbh->getSpecialQuery('tables') !== null): ?><input type="submit" class="button" name="list-tables" value="<?php echo _("List Tables") ?>" /> <?php endif; ?>
