@@ -4,10 +4,9 @@
  * the various caching backends.  It also provides functions for
  * checking in, retrieving, and flushing a cache.
  *
- * $Horde: framework/Cache/Cache.php,v 1.31.10.4 2005/07/03 05:11:46 selsky Exp $
+ * $Horde: framework/Cache/Cache.php,v 1.31.10.16 2008/01/02 11:30:24 jan Exp $
  *
- * Copyright 1999-2005 Anil Madhavapeddy <anil@recoil.org>
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -19,6 +18,28 @@
  */
 class Horde_Cache {
 
+    /**
+     * Cache parameters.
+     *
+     * @var array
+     */
+    var $_params = array();
+
+    /**
+     * Construct a new Horde_Cache object.
+     *
+     * @param array $params  Parameter array.
+     */
+    function Horde_Cache($params = array())
+    {
+        if (!isset($params['lifetime'])) {
+            $params['lifetime'] = isset($GLOBALS['conf']['cache']['default_lifetime'])
+                ? $GLOBALS['conf']['cache']['default_lifetime'] : 86400;
+        }
+
+        $this->_params = $params;
+    }
+
     /**
      * Attempts to retrieve a cached object and return it to the
      * caller.
@@ -40,12 +61,17 @@ class Horde_Cache {
      *
      * @abstract
      *
-     * @param string $key   Object ID used as the caching key.
-     * @param mixed  $data  Data to store in the cache.
+     * @param string $key        Object ID used as the caching key.
+     * @param mixed  $data       Data to store in the cache.
+     * @param integer $lifetime  Object lifetime - i.e. the time before the
+     *                           data becomes available for garbage
+     *                           collection.  If null use the default Horde GC
+     *                           time.  If 0 will not be GC'd.
+     *                           @since Horde 3.2
      *
      * @return boolean  True on success, false on failure.
      */
-    function set($key, $data)
+    function set($key, $data, $lifetime = null)
     {
         return true;
     }
@@ -58,11 +84,17 @@ class Horde_Cache {
      * @param string  $key       Object ID to query.
      * @param integer $lifetime  Lifetime of the object in seconds.
      *
-     * @return mixed  Cached data, or false if none was found.
+     * @return boolean  True if output or false if no object was found.
      */
     function output($key, $lifetime = 1)
     {
-        return false;
+        $data = $this->get($key, $lifetime);
+        if ($data === false) {
+            return false;
+        } else {
+            echo $data;
+            return true;
+        }
     }
 
     /**
@@ -90,11 +122,24 @@ class Horde_Cache {
      *
      * @return boolean  Success or failure.
      */
-    function expire($key, $lifetime = 1)
+    function expire($key)
     {
         return true;
     }
 
+    /**
+     * Determine the default lifetime for data.
+     *
+     * @private
+     * @since Horde 3.2
+     *
+     * @param mixed $lifetime  The lifetime to use or null for default.
+     */
+    function _getLifetime($lifetime)
+    {
+        return is_null($lifetime) ? $this->_params['lifetime'] : $lifetime;
+    }
+
     /**
      * Attempts to return a concrete Horde_Cache instance based on $driver.
      *
@@ -103,13 +148,13 @@ class Horde_Cache {
      *                       in $driver[0]/lib/Cache/ for the subclass
      *                       implementation named $driver[1].php.
      * @param array $params  A hash containing any additional
-     *                                configuration or connection parameters a
-     *                                subclass might need.
+     *                       configuration or connection parameters a subclass
+     *                       might need.
      *
      * @return Horde_Cache  The newly created concrete Horde_Cache instance, or
      *                      false on error.
      */
-    function &factory($driver, $params = array())
+    function factory($driver, $params = array())
     {
         if (is_array($driver)) {
             $app = $driver[0];
@@ -118,22 +163,25 @@ class Horde_Cache {
 
         $driver = basename($driver);
         if (empty($driver) || $driver == 'none') {
-            return $cache = &new Horde_Cache($params);
+            return new Horde_Cache($params);
         }
 
         if (!empty($app)) {
             include_once $app . '/lib/Cache/' . $driver . '.php';
-        } elseif (@file_exists(dirname(__FILE__) . '/Cache/' . $driver . '.php')) {
+        } elseif (file_exists(dirname(__FILE__) . '/Cache/' . $driver . '.php')) {
             include_once dirname(__FILE__) . '/Cache/' . $driver . '.php';
         } else {
-            @include_once 'Horde/Cache/' . $driver . '.php';
+            include_once 'Horde/Cache/' . $driver . '.php';
         }
+
         $class = 'Horde_Cache_' . $driver;
         if (class_exists($class)) {
-            return $cache = &new $class($params);
+            $cache = new $class($params);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $cache = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $cache;
     }
 
     /**
@@ -155,10 +203,7 @@ class Horde_Cache {
      * @param array $params  A hash containing any additional configuration or
      *                       connection parameters a subclass might need.
      *
-     * @since Horde 2.2
-     *
-     * @return Horde_Cache  The concrete Horde_Cache reference, or false on
-     *                      error.
+     * @return Horde_Cache  The concrete Horde_Cache reference, or PEAR_Error.
      */
     function &singleton($driver, $params = array())
     {
@@ -166,7 +211,7 @@ class Horde_Cache {
 
         $signature = serialize(array($driver, $params));
         if (empty($instances[$signature])) {
-            $instances[$signature] = &Horde_Cache::factory($driver, $params);
+            $instances[$signature] = Horde_Cache::factory($driver, $params);
         }
 
         return $instances[$signature];
