@@ -4,51 +4,64 @@
  * DataTree system.
  *
  * Required parameters:<pre>
- *   'phptype'   The database type (ie. 'pgsql', 'mysql', etc.).
- *   'charset'   The charset used by the database.</pre>
+ *   'phptype'      The database type (ie. 'pgsql', 'mysql', etc.).
+ *   'charset'      The charset used by the database.</pre>
  *
  * Optional parameters:<pre>
- *   'table'     The name of the data table in 'database'.
- *               DEFAULT: 'horde_datatree'</pre>
+ *   'table'        The name of the data table in 'database'.
+ *                  DEFAULT: 'horde_datatree'</pre>
  *
  * Required by some database implementations:<pre>
- *   'database'  The name of the database.
- *   'username'  The username with which to connect to the database.
- *   'password'  The password associated with 'username'.
- *   'hostspec'  The hostname of the database server.
- *   'protocol'  The communication protocol ('tcp', 'unix', etc.).
- *   'options'   Additional options to pass to the database.
- *   'port'      The port on which to connect to the database.
- *   'tty'       The TTY on which to connect to the database.</pre>
+ *   'database'     The name of the database.
+ *   'username'     The username with which to connect to the database.
+ *   'password'     The password associated with 'username'.
+ *   'hostspec'     The hostname of the database server.
+ *   'protocol'     The communication protocol ('tcp', 'unix', etc.).
+ *   'options'      Additional options to pass to the database.
+ *   'port'         The port on which to connect to the database.
+ *   'tty'          The TTY on which to connect to the database.</pre>
+ *
+ * Optional values when using separate reading and writing servers, for example
+ * in replication settings:<pre>
+ *   'splitread'   Boolean, whether to implement the separation or not.
+ *   'read'        Array containing the parameters which are different for
+ *                 the read database connection, currently supported
+ *                 only 'hostspec' and 'port' parameters.</pre>
  *
  * The table structure for the DataTree system is in
  * scripts/sql/horde_datatree.sql.
  *
- * $Horde: framework/DataTree/DataTree/sql.php,v 1.156.2.34 2007/12/06 18:10:39 jan Exp $
+ * $Horde: framework/DataTree/DataTree/sql.php,v 1.156.2.41 2008/07/12 10:37:45 jan Exp $
  *
- * Copyright 1999-2007 Stephane Huther <shuther@bigfoot.com>
- * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2005-2007 Jan Schneider <jan@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If
  * you did not receive this file, see
  * http://www.fsf.org/copyleft/lgpl.html.
  *
+ * @author  Stephane Huther <shuther1@free.fr>
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jan Schneider <jan@horde.org>
- * @author  Stephane Huther <shuther@bigfoot.com>
  * @since   Horde 2.1
  * @package Horde_DataTree
  */
 class DataTree_sql extends DataTree {
 
     /**
-     * Handle for the current database connection.
+     * Handle for the current database connection, used for reading.
      *
      * @var DB
      */
     var $_db;
 
+    /**
+     * Handle for the current database connection, used for writing. Defaults
+     * to the same handle as $_db if a separate write database is not required.
+     *
+     * @var DB
+     */
+    var $_write_db;
+
     /**
      * The number of copies of the horde_datatree_attributes table
      * that we need to join on in the current query.
@@ -57,17 +70,6 @@ class DataTree_sql extends DataTree {
      */
     var $_tableCount = 1;
 
-    /**
-     * Constructs a new SQL DataTree object.
-     *
-     * @param array $params  A hash containing connection parameters.
-     */
-    function DataTree_sql($params)
-    {
-        parent::DataTree($params);
-        $this->_connect();
-    }
-
     /**
      * Returns a list of all groups (root nodes) of the data tree.
      *
@@ -120,9 +122,6 @@ class DataTree_sql extends DataTree {
                                         $sortby_name,
                                         $sortby_key,
                                         $direction);
-        if (is_a($query, 'PEAR_Error')) {
-            return $query;
-        }
         if (empty($query)) {
             return true;
         }
@@ -149,9 +148,6 @@ class DataTree_sql extends DataTree {
     function _count($root = DATATREE_ROOT)
     {
         $query = $this->_buildLoadQuery($root, true, DATATREE_BUILD_COUNT);
-        if (is_a($query, 'PEAR_Error')) {
-            return $query;
-        }
         if (empty($query)) {
             return 0;
         }
@@ -188,7 +184,9 @@ class DataTree_sql extends DataTree {
 
         if (!empty($root) && $root != DATATREE_ROOT) {
             $parent_where = $this->_buildParentIds($root, $loadTree, 'c.');
-            if (!empty($parent_where) && !is_a($parent_where, 'PEAR_Error')) {
+            if (empty($parent_where)) {
+                return '';
+            } elseif (!is_a($parent_where, 'PEAR_Error')) {
                 $where = sprintf('%s AND (%s)', $where, $parent_where);
             }
         }
@@ -258,7 +256,7 @@ class DataTree_sql extends DataTree {
         $values = array($root,
                         $this->_params['group']);
 
-        Horde::logMessage('SQL Query by DataTree_sql::_buildParentIds(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        Horde::logMessage('SQL Query by DataTree_sql::_buildParentIds(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
         $root = $this->_db->getAssoc($query, false, $values);
         if (is_a($root, 'PEAR_Error') || !count($root)) {
             return '';
@@ -274,7 +272,7 @@ class DataTree_sql extends DataTree {
                 $pquery = ' OR ' . $join_name . 'datatree_id IN (' . str_replace(':', ', ', $ids) . ')';
             }
             if ($loadTree) {
-                $pquery .= ' OR ' . $join_name . 'datatree_parents = ' . $this->_db->quote(substr($pstring, 0, -2));
+                $pquery .= ' OR ' . $join_name . 'datatree_parents = ' . $this->_db->quote(substr($pstring, 0, -1));
             }
 
             if (!$first_time) {
@@ -345,7 +343,7 @@ class DataTree_sql extends DataTree {
             return $parents;
         }
         if (empty($parents)) {
-            return PEAR::raiseError(_("Not found."), null, null, null, 'DataTree ids ' . implode(', ', $ids) . ' not found.');
+            return PEAR::raiseError(_("Object not found."), null, null, null, 'DataTree ids ' . implode(', ', $ids) . ' not found.');
         }
 
         $ids = array();
@@ -372,7 +370,7 @@ class DataTree_sql extends DataTree {
                  ' AND group_uid = ? ORDER BY datatree_id';
         $values = array_merge($ids, array($this->_params['group']));
 
-        Horde::logMessage('SQL Query by DataTree_sql::_loadById(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        Horde::logMessage('SQL Query by DataTree_sql::_loadById(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
         $data = $this->_db->getAll($query, $values);
         if (is_a($data, 'PEAR_Error')) {
             return $data;
@@ -381,6 +379,93 @@ class DataTree_sql extends DataTree {
         return $this->set($data, $this->_params['charset']);
     }
 
+    /**
+     * Check for existance of an object in a backend-specific manner.
+     *
+     * @param string $object_name Object name to check for.
+     *
+     * @return boolean True if the object exists, false otherwise.
+     */
+    function _exists($object_name)
+    {
+        $query = 'SELECT datatree_id FROM ' . $this->_params['table'] .
+            ' WHERE group_uid = ? AND datatree_name = ? AND datatree_parents = ?';
+
+        $object_names = explode(':', $object_name);
+        $object_parents = '';
+        foreach ($object_names as $name) {
+            $values = array($this->_params['group'], $name, $object_parents);
+            Horde::logMessage('SQL Query by DataTree_sql::_exists(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+            $result = $this->_db->getOne($query, $values);
+            if (is_a($result, 'PEAR_Error') || !$result) {
+                return false;
+            }
+
+            $object_parents .= ':' . $result;
+        }
+
+        return true;
+    }
+
+    /**
+     * Look up a datatree id by name.
+     *
+     * @param string $name
+     *
+     * @return integer DataTree id
+     */
+    function _getId($name)
+    {
+        $query = 'SELECT datatree_id FROM ' . $this->_params['table']
+            . ' WHERE group_uid = ? AND datatree_name = ?'
+            . ' AND datatree_parents = ?';
+
+        $ids = array();
+        $parts = explode(':', $name);
+        foreach ($parts as $part) {
+            $result = $this->_db->getOne($query, array($this->_params['group'], $part, count($ids) ? ':' . implode(':', $ids) : ''));
+            if (is_a($result, 'PEAR_Error') || !$result) {
+                return null;
+            } else {
+                $ids[] = $result;
+            }
+        }
+
+        return (int)array_pop($ids);
+    }
+
+    /**
+     * Look up a datatree name by id.
+     *
+     * @param integer $id
+     *
+     * @return string DataTree name
+     */
+    function _getName($id)
+    {
+        $query = 'SELECT datatree_name FROM ' . $this->_params['table'] .
+            ' WHERE group_uid = ? AND datatree_id = ?';
+        $values = array($this->_params['group'], (int)$id);
+        Horde::logMessage('SQL Query by DataTree_sql::_getName(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+        $name = $this->_db->getOne($query, $values);
+        if (is_a($name, 'PEAR_Error')) {
+            return null;
+        } else {
+            $name = String::convertCharset($name, $this->_params['charset'],
+                                           NLS::getCharset());
+            // Get the parent names, if any.
+            $parent = $this->getParentById($id);
+            if ($parent && !is_a($parent, 'PEAR_Error') &&
+                $parent != DATATREE_ROOT) {
+                return $this->getName($parent) . ':' . $name;
+            } else {
+                return $name;
+            }
+        }
+    }
+
     /**
      * Returns a tree sorted by the specified attribute name and/or key.
      *
@@ -396,6 +481,8 @@ class DataTree_sql extends DataTree {
      * @param integer $direction   Sort direction:
      *                             0 - ascending
      *                             1 - descending
+     *
+     * @return array TODO
      */
     function getSortedTree($root, $loadTree = false, $sortby_name = null,
                            $sortby_key = null, $direction = 0)
@@ -407,6 +494,9 @@ class DataTree_sql extends DataTree {
                                         $sortby_key,
                                         $direction);
 
+        if (empty($query)) {
+            return array();
+        }
         return $this->_db->getAll($query);
     }
 
@@ -449,7 +539,7 @@ class DataTree_sql extends DataTree {
         }
 
         /* Get the next unique ID. */
-        $id = $this->_db->nextId($this->_params['table']);
+        $id = $this->_write_db->nextId($this->_params['table']);
         if (is_a($id, 'PEAR_Error')) {
             Horde::logMessage($id, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $id;
@@ -521,8 +611,8 @@ class DataTree_sql extends DataTree {
                         (int)$ser,
                         $parents);
 
-        Horde::logMessage('SQL Query by DataTree_sql::add(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-        $result = $this->_db->query($query, $values);
+        Horde::logMessage('SQL Query by DataTree_sql::add(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
@@ -587,12 +677,12 @@ class DataTree_sql extends DataTree {
             $query = sprintf('UPDATE %s SET datatree_order = datatree_order %s 1 WHERE group_uid = %s AND datatree_parents = %s AND datatree_order >= %s',
                              $this->_params['table'],
                              $direction,
-                             $this->_db->quote($this->_params['group']),
-                             $this->_db->quote($parent),
+                             $this->_write_db->quote($this->_params['group']),
+                             $this->_write_db->quote($parent),
                              is_null($order) ? 'NULL' : (int)$order) . $pquery;
 
             Horde::logMessage('SQL Query by DataTree_sql::reorder(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-            $result = $this->_db->query($query);
+            $result = $this->_write_db->query($query);
         } elseif (is_array($order)) {
             /* Multi update. */
             $query = 'SELECT COUNT(datatree_id)' .
@@ -602,7 +692,7 @@ class DataTree_sql extends DataTree {
             $values = array($this->_params['group'],
                             $parent);
 
-            Horde::logMessage('SQL Query by DataTree_sql::reorder(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            Horde::logMessage('SQL Query by DataTree_sql::reorder(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
             $result = $this->_db->getOne($query, $values);
             if (is_a($result, 'PEAR_Error')) {
@@ -617,8 +707,8 @@ class DataTree_sql extends DataTree {
                          ' SET datatree_order = ? WHERE datatree_id = ?';
                 $values = array($o_key, is_null($o_cid) ? NULL : (int)$o_cid);
 
-                Horde::logMessage('SQL Query by DataTree_sql::reorder(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-                $result = $this->_db->query($query, $values);
+                Horde::logMessage('SQL Query by DataTree_sql::reorder(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+                $result = $this->_write_db->query($query, $values);
                 if (is_a($result, 'PEAR_Error')) {
                     return $result;
                 }
@@ -646,8 +736,8 @@ class DataTree_sql extends DataTree {
         $values = array(is_null($order) ? NULL : (int)$order,
                         (int)$id);
 
-        Horde::logMessage('SQL Query by DataTree_sql::setOrder(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-        return $this->_db->query($query, $values);
+        Horde::logMessage('SQL Query by DataTree_sql::setOrder(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        return $this->_write_db->query($query, $values);
     }
 
     /**
@@ -670,7 +760,7 @@ class DataTree_sql extends DataTree {
         $values = array($this->_params['group'],
                         '%:' . (int)$id . '');
 
-        Horde::logMessage('SQL Query by DataTree_sql::remove(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        Horde::logMessage('SQL Query by DataTree_sql::remove(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
         $children = $this->_db->getAll($query, $values, DB_FETCHMODE_ASSOC);
 
         if (count($children)) {
@@ -692,8 +782,8 @@ class DataTree_sql extends DataTree {
                  ' WHERE datatree_id = ?';
         $values = array((int)$id);
 
-        Horde::logMessage('SQL Query by DataTree_sql::remove(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-        $result = $this->_db->query($query, $values);
+        Horde::logMessage('SQL Query by DataTree_sql::remove(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
@@ -702,8 +792,8 @@ class DataTree_sql extends DataTree {
                  ' WHERE datatree_id = ?';
         $values = array((int)$id);
 
-        Horde::logMessage('SQL Query by DataTree_sql::remove(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-        $result = $this->_db->query($query, $values);
+        Horde::logMessage('SQL Query by DataTree_sql::remove(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
@@ -751,8 +841,8 @@ class DataTree_sql extends DataTree {
                  ' WHERE datatree_id IN (?' . str_repeat(', ?', count($ids) - 1) . ')';
         $values = $ids;
 
-        Horde::logMessage('SQL Query by DataTree_sql::removeByIds(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-        $result = $this->_db->query($query, $values);
+        Horde::logMessage('SQL Query by DataTree_sql::removeByIds(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
@@ -761,8 +851,8 @@ class DataTree_sql extends DataTree {
                  ' WHERE datatree_id IN (?' . str_repeat(', ?', count($ids) - 1) . ')';
         $values = $ids;
 
-        Horde::logMessage('SQL Query by DataTree_sql::removeByIds(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-        return $this->_db->query($query, $values);
+        Horde::logMessage('SQL Query by DataTree_sql::removeByIds(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        return $this->_write_db->query($query, $values);
     }
 
     /**
@@ -791,7 +881,7 @@ class DataTree_sql extends DataTree {
                  ' WHERE datatree_name IN (?' . str_repeat(', ?', count($names) - 1) . ')';
         $values = $names;
 
-        Horde::logMessage('SQL Query by DataTree_sql::removeByNames(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        Horde::logMessage('SQL Query by DataTree_sql::removeByNames(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
         $ids = $this->_db->getCol($query, 0, $values);
         if (is_a($ids, 'PEAR_Error')) {
             return $ids;
@@ -827,7 +917,7 @@ class DataTree_sql extends DataTree {
                         $old_parent_path . ':' . $id . ':%',
                         (int)$id);
 
-        Horde::logMessage('SQL Query by DataTree_sql::move(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        Horde::logMessage('SQL Query by DataTree_sql::move(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
         $rowset = $this->_db->query($query, $values);
         if (is_a($rowset, 'PEAR_Error')) {
             return $rowset;
@@ -854,11 +944,11 @@ class DataTree_sql extends DataTree {
             }
             $query = sprintf('UPDATE %s SET datatree_parents = %s' . $oquery . ' WHERE datatree_id = %s',
                              $this->_params['table'],
-                             $this->_db->quote($ppath),
+                             $this->_write_db->quote($ppath),
                              (int)$row['datatree_id']);
 
             Horde::logMessage('SQL Query by DataTree_sql::move(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-            $result = $this->_db->query($query);
+            $result = $this->_write_db->query($query);
             if (is_a($result, 'PEAR_Error')) {
                 return $result;
             }
@@ -896,8 +986,8 @@ class DataTree_sql extends DataTree {
         $values = array(String::convertCharset($new_object_name, NLS::getCharset(), $this->_params['charset']),
                         (int)$id);
 
-        Horde::logMessage('SQL Query by DataTree_sql::rename(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-        $result = $this->_db->query($query, $values);
+        Horde::logMessage('SQL Query by DataTree_sql::rename(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        $result = $this->_write_db->query($query, $values);
 
         return is_a($result, 'PEAR_Error') ? $result : true;
     }
@@ -949,7 +1039,7 @@ class DataTree_sql extends DataTree {
                      ' WHERE datatree_id = ?';
             $values = array((int)$cid);
 
-            Horde::logMessage('SQL Query by DataTree_sql::getData(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            Horde::logMessage('SQL Query by DataTree_sql::getData(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
             $row = $this->_db->getRow($query, $values, DB_FETCHMODE_ASSOC);
 
             $data = Horde_Serialize::unserialize($row['datatree_data'],
@@ -1054,14 +1144,14 @@ class DataTree_sql extends DataTree {
         }
         list($query, $values) = $aq;
 
-        Horde::logMessage('SQL Query by DataTree_sql::countByAttributes(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        Horde::logMessage('SQL Query by DataTree_sql::countByAttributes(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         $result = $this->_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
         }
-        $row = &$result->fetchRow();
+        $row = $result->fetchRow();
         if (is_a($row, 'PEAR_Error')) {
             Horde::logMessage($row, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $row;
@@ -1098,6 +1188,46 @@ class DataTree_sql extends DataTree {
             return PEAR::raiseError('no criteria');
         }
 
+        // If there are top-level OR criteria, process one at a time
+        // and return any results as soon as they're found...but only if
+        // there is no LIMIT requested.
+        if ($count == 0 && $from == 0) {
+            foreach ($criteria as $key => $vals) {
+                if ($key == 'OR') {
+                    $rows = array();
+                    $num_or_statements = count($criteria[$key]);
+                    for ($i = 0; $i < $num_or_statements; $i++) {
+                        $criteria_or = $criteria['OR'][$i];
+                        list($query, $values) = $this->buildAttributeQuery(
+                                                    $criteria_or,
+                                                    $parent,
+                                                    $allLevels,
+                                                    $restrict,
+                                                    DATATREE_BUILD_SELECT,
+                                                    $sortby_name,
+                                                    $sortby_key,
+                                                    $direction);
+                        if ($count) {
+                            $query = $this->_db->modifyLimitQuery($query, $from, $count);
+                        }
+
+                        Horde::logMessage('SQL Query by DataTree_sql::getByAttributes(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+                        $result = $this->_db->query($query, $values);
+                        if (is_a($result, 'PEAR_Error')) {
+                            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+                            return $result;
+                        }
+                        while ($row = $result->fetchRow()) {
+                            $rows[$row[0]] = String::convertCharset($row[1], $this->_params['charset']);
+                        }
+                    }
+
+                    return $rows;
+                }
+            }
+        }
+        // Process AND or other complex queries.
         $aq = $this->buildAttributeQuery($criteria,
                                          $parent,
                                          $allLevels,
@@ -1109,22 +1239,22 @@ class DataTree_sql extends DataTree {
         if (is_a($aq, 'PEAR_Error')) {
             return $aq;
         }
+
         list($query, $values) = $aq;
 
-        Horde::logMessage('SQL Query by DataTree_sql::getByAttributes(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-
         if ($count) {
-            $result = $this->_db->limitQuery($query, $from, $count, $values);
-        } else {
-            $result = $this->_db->query($query, $values);
+            $query = $this->_db->modifyLimitQuery($query, $from, $count);
         }
 
+        Horde::logMessage('SQL Query by DataTree_sql::getByAttributes(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+        $result = $this->_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
         }
+
         $rows = array();
-        while ($row = &$result->fetchRow()) {
+        while ($row = $result->fetchRow()) {
             $rows[$row[0]] = String::convertCharset($row[1], $this->_params['charset']);
         }
 
@@ -1177,6 +1307,116 @@ class DataTree_sql extends DataTree {
         return $ordered_ids;
     }
 
+    /**
+     * Returns the number of all of the available values matching the
+     * given criteria. Either attribute_name or attribute_key MUST be
+     * supplied, and both MAY be supplied.
+     *
+     * @since Horde 3.2
+     *
+     * @see buildAttributeQuery()
+     *
+     * @param array   $criteria     The array of criteria.
+     * @param string  $parent       The parent node to start searching from.
+     * @param boolean $allLevels    Return all levels, or just the direct
+     *                              children of $parent? Defaults to all levels.
+     * @param string  $restrict     Only return attributes with the same
+     *                              attribute_name or attribute_id.
+     * @param string  $attribute_name  The name of the attribute.
+     * @param string  $attribute_key   The key value of the attribute.
+     */
+    function countValuesByAttributes($criteria, $parent = DATATREE_ROOT,
+                                     $allLevels = true, $restrict = 'name',
+                                     $key = null, $name = null)
+    {
+        if (!count($criteria)) {
+            return PEAR::raiseError('no criteria');
+        }
+
+        $aq = $this->buildAttributeQuery($criteria,
+                                         $parent,
+                                         $allLevels,
+                                         $restrict,
+                                         DATATREE_BUILD_VALUES_COUNT);
+
+        $aq[0] .= ' AND a.datatree_id = c.datatree_id';
+
+        if ($key !== null) {
+            $aq[0] .=  ' AND a.attribute_key = ?';
+            $aq[1][] = $key;
+        }
+
+        if ($name !== null) {
+            $aq[0] .=  ' AND a.attribute_name = ?';
+            $aq[1][] = $name;
+        }
+
+        return $this->_db->getOne($aq[0], $aq[1]);
+    }
+
+    /**
+     * Returns a list of all of the available values of the given criteria
+     * Either attribute_name or attribute_key MUST be
+     * supplied, and both MAY be supplied.
+     *
+     * @since Horde 3.2
+     *
+     * @see buildAttributeQuery()
+     *
+     * @param array   $criteria     The array of criteria.
+     * @param string  $parent       The parent node to start searching from.
+     * @param boolean $allLevels    Return all levels, or just the direct
+     *                              children of $parent? Defaults to all levels.
+     * @param string  $restrict     Only return attributes with the same
+     *                              attribute_name or attribute_id.
+     * @param integer $from         The object to start to fetching
+     * @param integer $count        The number of objects to fetch
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
+     * @param string  $attribute_name  The name of the attribute.
+     * @param string  $attribute_key   The key value of the attribute.
+     */
+    function getValuesByAttributes($criteria, $parent = DATATREE_ROOT,
+                                   $allLevels = true, $restrict = 'name', $from = 0,
+                                   $count = 0, $sortby_name = null,
+                                   $sortby_key = null, $direction = 0,
+                                   $key = null, $name = null)
+    {
+        if (!count($criteria)) {
+            return PEAR::raiseError('no criteria');
+        }
+
+        $aq = $this->buildAttributeQuery($criteria,
+                                         $parent,
+                                         $allLevels,
+                                         $restrict,
+                                         DATATREE_BUILD_VALUES,
+                                         $sortby_name,
+                                         $sortby_key,
+                                         $direction);
+
+        $aq[0] .=  ' AND a.datatree_id = c.datatree_id';
+
+        if ($key !== null) {
+            $aq[0] .=  ' AND a.attribute_key = ?';
+            $aq[1][] = $key;
+        }
+
+        if ($name !== null) {
+            $aq[0] .=  ' AND a.attribute_name = ?';
+            $aq[1][] = $name;
+        }
+
+        if ($count) {
+            $aq[0] = $this->_db->modifyLimitQuery($aq[0], $from, $count);
+        }
+
+        return $this->_db->getCol($aq[0], 0, $aq[1]);
+    }
+
     /**
      * Returns a list of all of the available values of the given attribute
      * name/key combination. Either attribute_name or attribute_key MUST be
@@ -1396,22 +1636,37 @@ class DataTree_sql extends DataTree {
 
         $joins = implode(' ', $joins);
         $pairs = implode(' ', $pairs);
-        $tail = sprintf('GROUP BY %s ORDER BY %s', $group_by, $order);
 
         switch ($operation) {
+
+        case DATATREE_BUILD_VALUES_COUNT:
+            $what = 'COUNT(DISTINCT(a.attribute_value))';
+            $from = ' ' . $this->_params['table_attributes'] . ' a, ' . $this->_params['table'];
+            $tail = '';
+            break;
+
+        case DATATREE_BUILD_VALUES:
+            $what = 'DISTINCT(a.attribute_value)';
+            $from = ' ' . $this->_params['table_attributes'] . ' a, ' . $this->_params['table'];
+            $tail = '';
+            break;
+
         case DATATREE_BUILD_COUNT:
             $what = 'COUNT(DISTINCT c.datatree_id)';
+            $from = $this->_params['table'];
             $tail = '';
             break;
 
         default:
             $what = 'c.datatree_id, c.datatree_name';
+            $from = $this->_params['table'];
+            $tail = sprintf('GROUP BY %s ORDER BY %s', $group_by, $order);
             break;
         }
 
         return array(sprintf('SELECT %s FROM %s c %s WHERE c.group_uid = ? AND %s %s %s %s',
                              $what,
-                             $this->_params['table'],
+                             $from,
                              $joins,
                              $query,
                              $levelQuery,
@@ -1531,22 +1786,22 @@ class DataTree_sql extends DataTree {
                      ' SET datatree_data = ? WHERE datatree_id = ?';
             $values = array(NULL, (int)$id);
 
-            Horde::logMessage('SQL Query by DataTree_sql::updateData(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-            $this->_db->query($query, $values);
+            Horde::logMessage('SQL Query by DataTree_sql::updateData(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            $this->_write_db->query($query, $values);
 
             /* Start a transaction. */
-            $this->_db->autoCommit(false);
+            $this->_write_db->autoCommit(false);
 
             /* Delete old attributes. */
             $query = 'DELETE FROM ' . $this->_params['table_attributes'] .
                      ' WHERE datatree_id = ?';
             $values = array((int)$id);
 
-            Horde::logMessage('SQL Query by DataTree_sql::updateData(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-            $result = $this->_db->query($query, $values);
+            Horde::logMessage('SQL Query by DataTree_sql::updateData(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            $result = $this->_write_db->query($query, $values);
             if (is_a($result, 'PEAR_Error')) {
-                $this->_db->rollback();
-                $this->_db->autoCommit(true);
+                $this->_write_db->rollback();
+                $this->_write_db->autoCommit(true);
                 return $result;
             }
 
@@ -1557,26 +1812,26 @@ class DataTree_sql extends DataTree {
             $query = 'INSERT INTO ' . $this->_params['table_attributes'] .
                      ' (datatree_id, attribute_name, attribute_key, attribute_value)' .
                      ' VALUES (?, ?, ?, ?)';
-            $statement = $this->_db->prepare($query);
+            $statement = $this->_write_db->prepare($query);
             foreach ($attributes as $attr) {
                 $values = array((int)$id,
                                 $attr['name'],
                                 $attr['key'],
                                 String::convertCharset($attr['value'], NLS::getCharset(), $this->_params['charset']));
 
-                Horde::logMessage('SQL Query by DataTree_sql::updateData(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
+                Horde::logMessage('SQL Query by DataTree_sql::updateData(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-                $result = $this->_db->execute($statement, $values);
+                $result = $this->_write_db->execute($statement, $values);
                 if (is_a($result, 'PEAR_Error')) {
-                    $this->_db->rollback();
-                    $this->_db->autoCommit(true);
+                    $this->_write_db->rollback();
+                    $this->_write_db->autoCommit(true);
                     return $result;
                 }
             }
 
             /* Commit the transaction, and turn autocommit back on. */
-            $result = $this->_db->commit();
-            $this->_db->autoCommit(true);
+            $result = $this->_write_db->commit();
+            $this->_write_db->autoCommit(true);
 
             return is_a($result, 'PEAR_Error') ? $result : true;
         } else {
@@ -1592,8 +1847,8 @@ class DataTree_sql extends DataTree {
                             (int)$ser,
                             (int)$id);
 
-            Horde::logMessage('SQL Query by DataTree_sql::updateData(): ' . $query, __FILE__, __LINE__, PEAR_LOG_DEBUG);
-            $result = $this->_db->query($query, $values);
+            Horde::logMessage('SQL Query by DataTree_sql::updateData(): ' . $query . ', ' . var_export($values, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            $result = $this->_write_db->query($query, $values);
 
             return is_a($result, 'PEAR_Error') ? $result : true;
         }
@@ -1604,46 +1859,56 @@ class DataTree_sql extends DataTree {
      *
      * @return boolean  True.
      */
-    function _connect()
+    function _init()
     {
-        Horde::assertDriverConfig($this->_params, 'storage',
+        Horde::assertDriverConfig($this->_params, 'sql',
             array('phptype', 'charset'),
             'DataTree SQL');
 
-        if (!isset($this->_params['database'])) {
-            $this->_params['database'] = '';
-        }
-        if (!isset($this->_params['username'])) {
-            $this->_params['username'] = '';
-        }
-        if (!isset($this->_params['password'])) {
-            $this->_params['password'] = '';
-        }
-        if (!isset($this->_params['hostspec'])) {
-            $this->_params['hostspec'] = '';
-        }
-        if (!isset($this->_params['table'])) {
-            $this->_params['table'] = 'horde_datatree';
-        }
-        if (!isset($this->_params['table_attributes'])) {
-            $this->_params['table_attributes'] = 'horde_datatree_attributes';
-        }
+        $default = array(
+            'database' => '',
+            'username' => '',
+            'password' => '',
+            'hostspec' => '',
+            'table' => 'horde_datatree',
+            'table_attributes' => 'horde_datatree_attributes',
+        );
+        $this->_params = array_merge($default, $this->_params);
 
         /* Connect to the SQL server using the supplied parameters. */
         require_once 'DB.php';
-        $this->_db = &DB::connect($this->_params,
-                                  array('persistent' => !empty($this->_params['persistent'])));
-        if (is_a($this->_db, 'PEAR_Error')) {
-            Horde::fatal($this->_db, __FILE__, __LINE__);
+        $this->_write_db = DB::connect($this->_params,
+                                       array('persistent' => !empty($this->_params['persistent'])));
+        if (is_a($this->_write_db, 'PEAR_Error')) {
+            return $this->_write_db;
         }
 
-        // Set DB portability options
-        switch ($this->_db->phptype) {
-        case 'mssql':
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
-            break;
-        default:
-            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+        // Set DB portability options.
+        $portability = DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS;
+        if ($this->_write_db->phptype == 'mssql') {
+            $portability |= DB_PORTABILITY_RTRIM;
+        }
+        $this->_write_db->setOption('portability', $portability);
+
+        /* Check if we need to set up the read DB connection
+         * seperately. */
+        if (!empty($this->_params['splitread'])) {
+            $params = array_merge($this->_params, $this->_params['read']);
+            $this->_db = DB::connect($params,
+                                     array('persistent' => !empty($params['persistent'])));
+            if (is_a($this->_db, 'PEAR_Error')) {
+                return $this->_db;
+            }
+
+            // Set DB portability options
+            $portability = DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS;
+            if ($this->_db->phptype == 'mssql') {
+                $portability |= DB_PORTABILITY_RTRIM;
+            }
+            $this->_db->setOption('portability', $portability);
+        } else {
+            /* Default to the same DB handle for reads. */
+            $this->_db = $this->_write_db;
         }
 
         return true;
