@@ -1,22 +1,21 @@
 <?php
 
 require_once 'Horde/DataTree.php';
+require_once 'Horde/Cache.php';
 
 /**
  * The Perms_datatree:: class provides a DataTree driver for the Horde
  * permissions system.
  *
- * $Horde: framework/Perms/Perms/datatree.php,v 1.6 2004/12/07 19:26:18 chuck Exp $
+ * $Horde: framework/Perms/Perms/datatree.php,v 1.6.2.21 2008/05/21 17:49:17 mrubinsk Exp $
  *
- * Copyright 2001-2004 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2004 Jan Schneider <jan@horde.org>
+ * Copyright 2001-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jan Schneider <jan@horde.org>
- * @version $Revision: 1.6 $
  * @since   Horde 3.0
  * @package Horde_Perms
  */
@@ -26,10 +25,17 @@ class Perms_datatree extends Perms {
      * Pointer to a DataTree instance to manage the different
      * permissions.
      *
-     * @var object DataTree $_datatree
+     * @var DataTree
      */
     var $_datatree;
 
+    /**
+     * Pointer to a Horde_Cache instance
+     *
+     * @var Horde_Cache
+     */
+    var $_cache;
+
     /**
      * Constructor.
      */
@@ -45,6 +51,9 @@ class Perms_datatree extends Perms {
         $this->_datatree = &DataTree::singleton($driver,
                                                 array_merge(Horde::getDriverConfig('datatree', $driver),
                                                             array('group' => 'horde.perms')));
+
+        $this->_cache = Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'],
+                                               Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));
     }
 
     /**
@@ -56,11 +65,21 @@ class Perms_datatree extends Perms {
      */
     function &newPermission($name)
     {
-        if (empty($name)) {
-            return PEAR::raiseError('Permission names must be non-empty');
+        $type = 'matrix';
+        $params = null;
+        if ($pos = strpos($name, ':')) {
+            $info = $this->getApplicationPermissions(substr($name, 0, $pos));
+            if (!is_a($info, 'PEAR_Error')) {
+                if (isset($info['type']) && isset($info['type'][$name])) {
+                    $type = $info['type'][$name];
+                }
+                if (isset($info['params']) && isset($info['params'][$name])) {
+                    $params = $info['params'][$name];
+                }
+            }
         }
 
-        $perm = &new DataTreeObject_Permission($name);
+        $perm = &new DataTreeObject_Permission($name, $type, $params);
         $perm->setDataTree($this->_datatree);
 
         return $perm;
@@ -82,7 +101,15 @@ class Perms_datatree extends Perms {
             return $permsCache[$name];
         }
 
-        $permsCache[$name] = $this->_datatree->getObject($name, 'DataTreeObject_Permission');
+        $perm = $this->_cache->get('perm_' . $name, $GLOBALS['conf']['cache']['default_lifetime']);
+        if ($perm === false) {
+            $perm = $this->_datatree->getObject($name, 'DataTreeObject_Permission');
+            $this->_cache->set('perm_' . $name, serialize($perm), $GLOBALS['conf']['cache']['default_lifetime']);
+            $permsCache[$name] = $perm;
+        } else {
+            $permsCache[$name] = unserialize($perm);
+        }
+
         return $permsCache[$name];
     }
 
@@ -94,7 +121,12 @@ class Perms_datatree extends Perms {
      */
     function &getPermissionById($cid)
     {
-        return $this->_datatree->getObjectById($cid, 'DataTreeObject_Permission');
+        if ($cid == PERMS_ROOT) {
+            $object = &$this->newPermission(PERMS_ROOT);
+        } else {
+            $object = &$this->_datatree->getObjectById($cid, 'DataTreeObject_Permission');
+        }
+        return $object;
     }
 
     /**
@@ -110,6 +142,13 @@ class Perms_datatree extends Perms {
             return PEAR::raiseError('Permissions must be DataTreeObject_Permission objects or extend that class.');
         }
 
+        $name = $perm->getName();
+        if (empty($name)) {
+            return PEAR::raiseError('Permission names must be non-empty');
+        }
+        $this->_cache->expire('perm_' . $name);
+        $this->_cache->expire('perm_exists_' . $name);
+
         return $this->_datatree->add($perm);
     }
 
@@ -125,86 +164,13 @@ class Perms_datatree extends Perms {
             return PEAR::raiseError('Permissions must be DataTreeObject_Permission objects or extend that class.');
         }
 
-        return $this->_datatree->remove($perm->getName(), $force);
-    }
-
-    /**
-     * Finds out what rights the given user has to this object.
-     *
-     * @param mixed $permission  The full permission name of the object to
-     *                           check the permissions of, or the
-     *                           DataTreeObject_Permission object.
-     * @param string $user       The user to check for. Defaults to the current
-     *                           user.
-     * @param string $creator    The user who created the event.
-     *
-     * @return mixed  A bitmask of permissions the user has, false if there
-     *                are none.
-     */
-    function getPermissions($permission, $user = null, $creator = null)
-    {
-        if (!is_a($permission, 'DataTreeObject_Permission')) {
-            $permission = &$this->getPermission($permission);
-            if (is_a($permission, 'PEAR_Error')) {
-                Horde::logMessage($permission, __FILE__, __LINE__, PEAR_LOG_INFO);
-                return false;
-            }
+        $keys = $this->_datatree->get(DATATREE_FORMAT_FLAT, $perm->name, true);
+        foreach ($keys as $key) {
+            $this->_cache->expire('perm_' . $key);
+            $this->_cache->expire('perm_exists_' . $key);
         }
 
-        if (is_null($user)) {
-            $user = Auth::getAuth();
-        }
-
-        // If this is a guest user, only check guest permissions.
-        if (empty($user)) {
-            return $permission->getGuestPermissions();
-        }
-
-        // If $creator was specified, check creator permissions.
-        if (!is_null($creator)) {
-            // If the user is the creator of the event see if there
-            // are creator permissions.
-            if (strlen($user) && $user === $creator &&
-                ($perms = $permission->getCreatorPermissions()) !== null) {
-                return $perms;
-            }
-        }
-
-        // Check user-level permissions.
-        $userperms = $permission->getUserPermissions();
-        if (isset($userperms[$user])) {
-            return $userperms[$user];
-        }
-
-        // If no user permissions are found, try group permissions.
-        if (isset($permission->data['groups']) &&
-            is_array($permission->data['groups']) &&
-            count($permission->data['groups'])) {
-            require_once 'Horde/Group.php';
-            $groups = &Group::singleton();
-
-            $composite_perm = null;
-            foreach ($permission->data['groups'] as $group => $perm) {
-                if ($groups->userIsInGroup($user, $groups->getGroupName($group))) {
-                    if (is_null($composite_perm)) {
-                        $composite_perm = 0;
-                    }
-                    $composite_perm |= $perm;
-                }
-            }
-
-            if (!is_null($composite_perm)) {
-                return $composite_perm;
-            }
-        }
-
-        // If there are default permissions, return them.
-        if (($perms = $permission->getDefaultPermissions()) !== null) {
-            return $perms;
-        }
-
-        // Otherwise, deny all permissions to the object.
-        return false;
+        return $this->_datatree->remove($perm->name, $force);
     }
 
     /**
@@ -220,22 +186,6 @@ class Perms_datatree extends Perms {
         return $this->_datatree->getId($permission->getName());
     }
 
-    /**
-     * Finds out if the user has the specified rights to the given object.
-     *
-     * @param string $permission  The permission to check.
-     * @param string $user        The user to check for.
-     * @param int $perm           The permission level that needs to be checked
-     *                            for.
-     * @param string $creator     The creator of the event
-     *
-     * @return boolean  True if the user has the specified permissions.
-     */
-    function hasPermission($permission, $user, $perm, $creator = null)
-    {
-        return ($this->getPermissions($permission, $user, $creator) & $perm);
-    }
-
     /**
      * Checks if a permission exists in the system.
      *
@@ -245,7 +195,14 @@ class Perms_datatree extends Perms {
      */
     function exists($permission)
     {
-        return $this->_datatree->exists($permission);
+        $key = 'perm_exists_' . $permission;
+        $exists = $this->_cache->get($key, $GLOBALS['conf']['cache']['default_lifetime']);
+        if ($exists === false) {
+            $exists = $this->_datatree->exists($permission);
+            $this->_cache->set($key, $exists);
+        }
+
+        return (bool)$exists;
     }
 
     /**
@@ -260,6 +217,20 @@ class Perms_datatree extends Perms {
         return $this->_datatree->getParents($child);
     }
 
+    /**
+     * Returns a child's direct parent ID.
+     *
+     * @param mixed $child  Either the object, an array containing the
+     *                      path elements, or the object name for which
+     *                      to look up the parent's ID.
+     *
+     * @return mixed  The unique ID of the parent or PEAR_Error on error.
+     */
+    function getParent($child)
+    {
+        return $this->_datatree->getParent($child);
+    }
+
     /**
      * Returns all permissions of the system in a tree format.
      *
@@ -267,59 +238,95 @@ class Perms_datatree extends Perms {
      */
     function &getTree()
     {
-        return $this->_datatree->get(DATATREE_FORMAT_FLAT, DATATREE_ROOT, true);
+        $tree = $this->_datatree->get(DATATREE_FORMAT_FLAT, PERMS_ROOT, true);
+        return $tree;
     }
 
 }
 
 /**
- * Extension of the DataTreeObject class for storing Permission
- * information in the DataTree driver. If you want to store
- * specialized Permission information, you should extend this class
- * instead of extending DataTreeObject directly.
+ * Extension of the DataTreeObject class for storing Permission information in
+ * the DataTree driver. If you want to store specialized Permission
+ * information, you should extend this class instead of extending
+ * DataTreeObject directly.
+ *
+ * @TODO This class duplicates most of the functionality of the
+ * Horde_Permission class. However, because for BC/DataTree reasons it
+ * must extend DataTreeObject, we can't remove these methods yet.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jan Schneider <jan@horde.org>
- * @version $Revision: 1.6 $
  * @since   Horde 2.1
  * @package Horde_Perms
  */
 class DataTreeObject_Permission extends DataTreeObject {
 
     /**
-     * The DataTreeObject_Permission constructor. Just makes sure to
-     * call the parent constructor so that the perm's name is set
-     * properly.
+     * The DataTreeObject_Permission constructor. Just makes sure to call the
+     * parent constructor so that the perm's name is set properly.
      *
-     * @param string $name The name of the perm.
+     * @param string $name   The name of the perm.
+     * @param string $type   The permission type.
+     * @param array $params  A hash with any parameters that the permission
+     *                       type needs.
      */
-    function DataTreeObject_Permission($name)
+    function DataTreeObject_Permission($name, $type = 'matrix', $params = null)
     {
         parent::DataTreeObject($name);
+        $this->data['type'] = $type;
+        if (is_array($params)) {
+            $this->data['params'] = $params;
+        }
     }
 
     /**
-     * Update the permissions based on data passed in the array.
+     * Gets one of the attributes of the object, or null if it isn't defined.
+     *
+     * @param string $attribute  The attribute to get.
+     *
+     * @return mixed  The value of the attribute, or null.
+     */
+    function get($attribute)
+    {
+        $value = parent::get($attribute);
+        if (is_null($value) && $attribute == 'type') {
+            $value = 'matrix';
+        }
+        return $value;
+    }
+
+    /**
+     * Updates the permissions based on data passed in the array.
      *
      * @param array $perms  An array containing the permissions which are to be
      *                      updated.
      */
     function updatePermissions($perms)
     {
-        /* Array of permission types to iterate through. */
-        $perm_types = Perms::getPermsArray();
+        $type = $this->get('type');
+
+        if ($type == 'matrix') {
+            /* Array of permission types to iterate through. */
+            $perm_types = Perms::getPermsArray();
+        }
 
         foreach ($perms as $perm_class => $perm_values) {
             switch ($perm_class) {
             case 'default':
             case 'guest':
             case 'creator':
-                foreach ($perm_types as $val => $label) {
-                    if (!empty($perm_values[$val])) {
-                        $this->setPerm($perm_class, $val, false);
-                    } else {
-                        $this->unsetPerm($perm_class, $val, false);
+                if ($type == 'matrix') {
+                    foreach ($perm_types as $val => $label) {
+                        if (!empty($perm_values[$val])) {
+                            $this->setPerm($perm_class, $val, false);
+                        } else {
+                            $this->unsetPerm($perm_class, $val, false);
+                        }
                     }
+                } elseif (!empty($perm_values)) {
+                    $this->setPerm($perm_class, $perm_values, false);
+                } else {
+                    $this->unsetPerm($perm_class, null, false);
                 }
                 break;
 
@@ -329,7 +336,9 @@ class DataTreeObject_Permission extends DataTreeObject {
                 /* Figure out what names that are stored in this permission
                  * class have not been submitted for an update, ie. have been
                  * removed entirely. */
-                $current_names = isset($this->data[$permId['class']]) ? array_keys($this->data[$permId['class']]) : array();
+                $current_names = isset($this->data[$permId['class']])
+                    ? array_keys($this->data[$permId['class']])
+                    : array();
                 $updated_names = array_keys($perm_values);
                 $removed_names = array_diff($current_names, $updated_names);
 
@@ -347,14 +356,21 @@ class DataTreeObject_Permission extends DataTreeObject {
                 foreach ($perm_values as $name => $name_values) {
                     $permId['name'] = $name;
 
-                    foreach ($perm_types as $val => $label) {
-                        if (!empty($name_values[$val])) {
-                            $this->setPerm($permId, $val, false);
-                        } else {
-                            $this->unsetPerm($permId, $val, false);
+                    if ($type == 'matrix') {
+                        foreach ($perm_types as $val => $label) {
+                            if (!empty($name_values[$val])) {
+                                $this->setPerm($permId, $val, false);
+                            } else {
+                                $this->unsetPerm($permId, $val, false);
+                            }
                         }
+                    } elseif (!empty($name_values)) {
+                        $this->setPerm($permId, $name_values, false);
+                    } else {
+                        $this->unsetPerm($permId, null, false);
                     }
                 }
+
                 break;
             }
         }
@@ -369,13 +385,15 @@ class DataTreeObject_Permission extends DataTreeObject {
             if (empty($permId['name'])) {
                 return;
             }
-            if (isset($this->data[$permId['class']][$permId['name']])) {
+            if ($this->get('type') == 'matrix' &&
+                isset($this->data[$permId['class']][$permId['name']])) {
                 $this->data[$permId['class']][$permId['name']] |= $permission;
             } else {
                 $this->data[$permId['class']][$permId['name']] = $permission;
             }
         } else {
-            if (isset($this->data[$permId])) {
+            if ($this->get('type') == 'matrix' &&
+                isset($this->data[$permId])) {
                 $this->data[$permId] |= $permission;
             } else {
                 $this->data[$permId] = $permission;
@@ -396,18 +414,32 @@ class DataTreeObject_Permission extends DataTreeObject {
             if (empty($permId['name'])) {
                 return;
             }
-            if (isset($this->data[$permId['class']][$permId['name']])) {
-                $this->data[$permId['class']][$permId['name']] &= ~$permission;
-                if (empty($this->data[$permId['class']][$permId['name']])) {
-                    unset($this->data[$permId['class']][$permId['name']]);
+            if ($this->get('type') == 'matrix') {
+                if (isset($this->data[$permId['class']][$permId['name']])) {
+                    $this->data[$permId['class']][$permId['name']] &= ~$permission;
+                    if (empty($this->data[$permId['class']][$permId['name']])) {
+                        unset($this->data[$permId['class']][$permId['name']]);
+                    }
+                    if ($update) {
+                        $this->save();
+                    }
                 }
+            } else {
+                unset($this->data[$permId['class']][$permId['name']]);
                 if ($update) {
                     $this->save();
                 }
             }
         } else {
-            if (isset($this->data[$permId])) {
-                $this->data[$permId] &= ~$permission;
+            if ($this->get('type') == 'matrix') {
+                if (isset($this->data[$permId])) {
+                    $this->data[$permId] &= ~$permission;
+                    if ($update) {
+                        $this->save();
+                    }
+                }
+            } else {
+                unset($this->data[$permId]);
                 if ($update) {
                     $this->save();
                 }
@@ -416,19 +448,21 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Give a user additional permissions to this object.
+     * Grants a user additional permissions to this object.
      *
-     * @param string   $user        The user to grant additional permissions to.
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to add.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param string $user          The user to grant additional permissions
+     *                              to.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to add.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function addUserPermission($user, $permission, $update = true)
     {
         if (empty($user)) {
             return;
         }
-        if (isset($this->data['users'][$user])) {
+        if ($this->get('type') == 'matrix' &&
+            isset($this->data['users'][$user])) {
             $this->data['users'][$user] |= $permission;
         } else {
             $this->data['users'][$user] = $permission;
@@ -440,15 +474,16 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Grant guests additional permissions to this object.
+     * Grants guests additional permissions to this object.
      *
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to add.
-     * @param boolean  $update     (optional) Whether to automatically update the
-     *                             backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to add.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function addGuestPermission($permission, $update = true)
     {
-        if (isset($this->data['guest'])) {
+        if ($this->get('type') == 'matrix' &&
+            isset($this->data['guest'])) {
             $this->data['guest'] |= $permission;
         } else {
             $this->data['guest'] = $permission;
@@ -460,15 +495,16 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Grant creators additional permissions to this object.
+     * Grants creators additional permissions to this object.
      *
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to add.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to add.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function addCreatorPermission($permission, $update = true)
     {
-        if (isset($this->data['creator'])) {
+        if ($this->get('type') == 'matrix' &&
+            isset($this->data['creator'])) {
             $this->data['creator'] |= $permission;
         } else {
             $this->data['creator'] = $permission;
@@ -480,15 +516,16 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Grant additional default permissions to this object.
+     * Grants additional default permissions to this object.
      *
-     * @param integer $permission  The permission (PERMS_DELE, etc.) to add.
-     * @param boolean $update      (optional) Whether to automatically update the
-     *                             backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to add.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function addDefaultPermission($permission, $update = true)
     {
-        if (isset($this->data['default'])) {
+        if ($this->get('type') == 'matrix' &&
+            isset($this->data['default'])) {
             $this->data['default'] |= $permission;
         } else {
             $this->data['default'] = $permission;
@@ -500,12 +537,13 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Give a group additional permissions to this object.
+     * Grants a group additional permissions to this object.
      *
-     * @param integer  $groupId    The id of the group to grant additional permissions to.
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to add.
-     * @param boolean  $update     (optional) Whether to automatically update the
-     *                             backend. Defaults to true.
+     * @param integer $groupId      The id of the group to grant additional
+     *                              permissions to.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to add.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function addGroupPermission($groupId, $permission, $update = true)
     {
@@ -513,7 +551,8 @@ class DataTreeObject_Permission extends DataTreeObject {
             return;
         }
 
-        if (isset($this->data['groups'][$groupId])) {
+        if ($this->get('type') == 'matrix' &&
+            isset($this->data['groups'][$groupId])) {
             $this->data['groups'][$groupId] |= $permission;
         } else {
             $this->data['groups'][$groupId] = $permission;
@@ -525,75 +564,82 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Remove a permission that a user currently has on this object.
+     * Removes a permission that a user currently has on this object.
      *
-     * @param string   $user         The user to remove the permission from.
-     * @param constant $permission   The permission (PERMS_DELE, etc.) to
-     *                               remove.
-     * @param optional bool $update  Whether to automatically update the
-     *                               backend. Defaults to true.
+     * @param string $user          The user to remove the permission from.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to
+     *                              remove.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function removeUserPermission($user, $permission, $update = true)
     {
-        if (empty($user)) {
+        if (empty($user) || !isset($this->data['users'][$user])) {
             return;
         }
-        if (isset($this->data['users'][$user])) {
+
+        if ($this->get('type') == 'matrix') {
             $this->data['users'][$user] &= ~$permission;
             if (empty($this->data['users'][$user])) {
                 unset($this->data['users'][$user]);
             }
-            if ($update) {
-                $this->save();
-            }
+        } else {
+            unset($this->data['users'][$user]);
+        }
+
+        if ($update) {
+            $this->save();
         }
     }
 
     /**
-     * Remove a permission that guests currently have on this object.
+     * Removes a permission that guests currently have on this object.
      *
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to remove.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to
+     *                              remove.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function removeGuestPermission($permission, $update = true)
     {
-        if (isset($this->data['guest'])) {
+        if (!isset($this->data['guest'])) {
+            return;
+        }
+
+        if ($this->get('type') == 'matrix') {
             $this->data['guest'] &= ~$permission;
             if ($update) {
                 $this->save();
             }
+        } else {
+            unset($this->data['guest']);
+            if ($update) {
+                $this->save();
+            }
         }
     }
 
     /**
-     * Remove a permission that creators currently have on this object.
+     * Removes a permission that creators currently have on this object.
      *
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to remove.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to
+     *                              remove.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
     function removeCreatorPermission($permission, $update = true)
     {
-        if (isset($this->data['creator'])) {
+        if (!isset($this->data['creator'])) {
+            return;
+        }
+
+        if ($this->get('type') == 'matrix') {
             $this->data['creator'] &= ~$permission;
             if ($update) {
                 $this->save();
             }
-        }
-    }
-
-    /**
-     * Remove a default permission on this object.
-     *
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to remove.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
-     */
-    function removeDefaultPermission($permission, $update = true)
-    {
-        if (isset($this->data['default'])) {
-            $this->data['default'] &= ~$permission;
+        } else {
+            unset($this->data['creator']);
             if ($update) {
                 $this->save();
             }
@@ -601,20 +647,49 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Remove a permission that a group currently has on this object.
+     * Removes a default permission on this object.
      *
-     * @param integer  $groupId     The id of the group to remove the permission from.
-     * @param constant $permission  The permission (PERMS_DELE, etc.) to remove.
-     * @param boolean  $update      (optional) Whether to automatically update the
-     *                              backend. Defaults to true.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to
+     *                              remove.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
      */
-    function removeGroupPermission($groupId, $permission, $update = true)
+    function removeDefaultPermission($permission, $update = true)
     {
-        if (empty($groupId)) {
+        if (!isset($this->data['default'])) {
             return;
         }
 
-        if (isset($this->data['groups'][$groupId])) {
+        if ($this->get('type') == 'matrix') {
+            $this->data['default'] &= ~$permission;
+            if ($update) {
+                $this->save();
+            }
+        } else {
+            unset($this->data['default']);
+            if ($update) {
+                $this->save();
+            }
+        }
+    }
+
+    /**
+     * Removes a permission that a group currently has on this object.
+     *
+     * @param integer $groupId      The id of the group to remove the
+     *                              permission from.
+     * @param constant $permission  The permission (PERMS_DELETE, etc.) to
+     *                              remove.
+     * @param boolean $update       Whether to automatically update the
+     *                              backend.
+     */
+    function removeGroupPermission($groupId, $permission, $update = true)
+    {
+        if (empty($groupId) || !isset($this->data['groups'][$groupId])) {
+            return;
+        }
+
+        if ($this->get('type') == 'matrix') {
             $this->data['groups'][$groupId] &= ~$permission;
             if (empty($this->data['groups'][$groupId])) {
                 unset($this->data['groups'][$groupId]);
@@ -622,14 +697,19 @@ class DataTreeObject_Permission extends DataTreeObject {
             if ($update) {
                 $this->save();
             }
+        } else {
+            unset($this->data['groups'][$groupId]);
+            if ($update) {
+                $this->save();
+            }
         }
     }
 
     /**
-     * Get an array of all user permissions on this object.
+     * Returns an array of all user permissions on this object.
      *
-     * @param optional integer $perm  List only users with this permission
-     *                                level. Defaults to all users.
+     * @param integer $perm  List only users with this permission level.
+     *                       Defaults to all users.
      *
      * @return array  All user permissions for this object, indexed by user.
      */
@@ -651,7 +731,7 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Get the guest permissions on this object.
+     * Returns the guest permissions on this object.
      *
      * @return integer  The guest permissions on this object.
      */
@@ -663,7 +743,7 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Get the creator permissions on this object.
+     * Returns the creator permissions on this object.
      *
      * @return integer  The creator permissions on this object.
      */
@@ -675,7 +755,7 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Get the default permissions on this object.
+     * Returns the default permissions on this object.
      *
      * @return integer  The default permissions on this object.
      */
@@ -687,16 +767,17 @@ class DataTreeObject_Permission extends DataTreeObject {
     }
 
     /**
-     * Get an array of all group permissions on this object.
+     * Returns an array of all group permissions on this object.
      *
-     * @param optional integer $perm  List only users with this permission
-     *                                level. Defaults to all users.
+     * @param integer $perm  List only users with this permission level.
+     *                       Defaults to all users.
      *
      * @return array  All group permissions for this object, indexed by group.
      */
     function getGroupPermissions($perm = null)
     {
-        if (!isset($this->data['groups']) || !is_array($this->data['groups'])) {
+        if (!isset($this->data['groups']) ||
+            !is_array($this->data['groups'])) {
             return array();
         } elseif (!$perm) {
             return $this->data['groups'];
@@ -711,4 +792,24 @@ class DataTreeObject_Permission extends DataTreeObject {
         }
     }
 
+    /**
+     * Saves any changes to this object to the backend permanently. New objects
+     * are added instead.
+     *
+     * @return boolean|PEAR_Error  PEAR_Error on failure.
+     */
+    function save()
+    {
+
+        $name = $this->getName();
+        if (empty($name)) {
+            return PEAR::raiseError('Permission names must be non-empty');
+        }
+        parent::save();
+
+        $cache = Horde_Cache::singleton($GLOBALS['conf']['cache']['driver'], Horde::getDriverConfig('cache', $GLOBALS['conf']['cache']['driver']));
+        $cache->expire('perm_' . $name);
+        $cache->expire('perm_exists_' . $name);
+    }
+
 }
