@@ -12,14 +12,14 @@ require_once 'Horde/String.php';
  * wrap    -- Whether to wrap the text or not.
  * </pre>
  *
- * $Horde: framework/Text_Filter/Filter/html2text.php,v 1.4.10.14 2007/01/02 13:54:43 jan Exp $
+ * $Horde: framework/Text_Filter/Filter/html2text.php,v 1.4.10.20 2008/09/25 09:00:59 jan Exp $
  *
- * Copyright 2003-2007 Jon Abernathy <jon@chuggnutt.com>
+ * Copyright 2003-2004 Jon Abernathy <jon@chuggnutt.com>
  * Original source: http://www.chuggnutt.com/html2text.php
- * Copyright 2004-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL). If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jon Abernathy <jon@chuggnutt.com>
  * @author  Jan Schneider <jan@horde.org>
@@ -73,7 +73,7 @@ class Text_Filter_html2text extends Text_Filter {
             '/^\s*(.*?)\s*$/m' => '\1',
 
             // Normalize <br>.
-            '/<br[^>]*>([^\n])/i' => "<br>\n\\1",
+            '/<br[^>]*>([^\n]*)\n/i' => "<br>\\1",
 
             // Newlines and tabs.
             '/[\n\t]+/' => ' ',
@@ -94,10 +94,10 @@ class Text_Filter_html2text extends Text_Filter {
             '/<h[456][^>]*>(.+?)<\/h[456]> ?/ie' => 'ucwords("\n\n" . \'\1\' . "\n\n")',
 
             // <p>
-            '/<p[^>]*> ?/i' => "\n\n  ",
+            '/<p[^>]*> ?/i' => "\n\n",
 
-            // <br>
-            '/<br[^>]*> ?/i' => "\n",
+            // <br>/<div>
+            '/<(br|div)[^>]*> ?/i' => "\n",
 
             // <b>
             '/<b[^>]*>(.+?)<\/b>/ie' => 'strtoupper(\'\1\')',
@@ -162,16 +162,21 @@ class Text_Filter_html2text extends Text_Filter {
     {
         global $_html2text_state;
 
-        /* Quote block quotes. */
+        /* Quote block quotes and strip HTML tags inside quotes.
+         * NB: The quotes in the replace string have to be exactly this way,
+         * otherwise preg_replace() would run the matched text through
+         * addslashes(). */
         $text = preg_replace('/<blockquote type="?cite"?>\n?(.*?)\n?<\/blockquote>\n?/ies',
-                             '"\n\n" . $this->_quote(\'$1\') . "\n\n"',
+                             "\"\n\n\" . \$this->_quote(strip_tags('$1')) . \"\n\n\"",
                              $text);
 
         /* Strip any other HTML tags. */
         $text = strip_tags($text);
 
-        /* Convert html entities. */
-        $text = @html_entity_decode($text, ENT_QUOTES, $this->_params['charset']);
+        /* Convert HTML entities. */
+        $trans = array_flip(get_html_translation_table(HTML_ENTITIES));
+        $trans = String::convertCharset($trans, 'ISO-8859-1', $this->_params['charset']);
+        $text = strtr($text, $trans);
 
         /* Bring down number of empty lines to 2 max. */
         $text = preg_replace("/\n[[:space:]]+\n/", "\n\n", $text);
@@ -255,6 +260,36 @@ class Text_Filter_html2text extends Text_Filter {
     {
         global $_html2text_state;
 
+        if ($link == strip_tags($display)) {
+            return $display;
+        }
+        $parsed_link = parse_url($link);
+        $parsed_display = parse_url(strip_tags($display));
+        if (isset($parsed_link['path'])) {
+            $parsed_link['path'] = trim($parsed_link['path'], '/');
+            if (!strlen($parsed_link['path'])) {
+                unset($parsed_link['path']);
+            }
+        }
+        if (isset($parsed_display['path'])) {
+            $parsed_display['path'] = trim($parsed_display['path'], '/');
+            if (!strlen($parsed_display['path'])) {
+                unset($parsed_display['path']);
+            }
+        }
+        if (((!isset($parsed_link['host']) &&
+              !isset($parsed_display['host'])) ||
+             (isset($parsed_link['host']) &&
+              isset($parsed_display['host']) &&
+              $parsed_link['host'] == $parsed_display['host'])) &&
+            ((!isset($parsed_link['path']) &&
+              !isset($parsed_display['path'])) ||
+             (isset($parsed_link['path']) &&
+              isset($parsed_display['path']) &&
+              $parsed_link['path'] == $parsed_display['path']))) {
+            return $display;
+        }
+
         $_html2text_state['linkCount']++;
         $_html2text_state['linkList'] .= '[' . $_html2text_state['linkCount'] . "] $link\n";
         return $display . '[' . $_html2text_state['linkCount'] . ']';
