@@ -6,9 +6,9 @@
  * people *KNOW* HTML is a security hole, clean up what we can, and leave it
  * at that.
  *
- * $Horde: framework/Text_Filter/Filter/xss.php,v 1.1.2.8.2.3 2008/09/10 09:14:42 jan Exp $
+ * $Horde: framework/Text_Filter/Filter/xss.php,v 1.1.2.15 2008/09/24 03:33:57 chuck Exp $
  *
- * Copyright 2004-2007 Jan Schneider <jan@horde.org>
+ * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -26,7 +26,8 @@ class Text_Filter_xss extends Text_Filter {
      */
     var $_params = array('body_only' => true,
                          'replace' => 'XSSCleaned',
-                         'strip_styles' => true);
+                         'strip_styles' => true,
+                         'strip_style_attributes' => true);
 
     /**
      * Returns a hash with replace patterns.
@@ -191,7 +192,9 @@ class Text_Filter_xss extends Text_Filter {
 
         /* Comment out style/link tags. */
         if ($this->_params['strip_styles']) {
-            $patterns['/\s+style\s*=/i'] = ' ' . $this->_params['replace'] . '=';
+            if ($this->_params['strip_style_attributes']) {
+                $patterns['/\s+style\s*=/i'] = ' ' . $this->_params['replace'] . '=';
+            }
             $patterns['|<style[^>]*>(?:\s*<\!--)*|i'] = '<!--';
             $patterns['|(?:-->\s*)*</style>|i'] = '-->';
             $patterns['|(<link[^>]*>)|i'] = '<!-- $1 -->';
@@ -225,7 +228,7 @@ class Text_Filter_xss extends Text_Filter {
     }
 
     /**
-     * Executes any code necessaray after applying the filter patterns.
+     * Executes any code necessary after applying the filter patterns.
      *
      * @param string $text  The text after the filtering.
      *
