@@ -7,13 +7,14 @@
  * String package
  */
 include_once 'Horde/String.php';
+include_once 'Horde/NLS.php';
 
 /**
  * Class representing iCalendar files.
  *
- * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.46 2007/03/15 18:36:27 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.69 2008/08/29 13:42:25 jan Exp $
  *
- * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -84,7 +85,7 @@ class Horde_iCalendar {
         $type = String::lower($type);
         $class = 'Horde_iCalendar_' . $type;
         if (!class_exists($class)) {
-            include_once 'Horde/iCalendar/' . $type . '.php';
+            include 'Horde/iCalendar/' . $type . '.php';
         }
         if (class_exists($class)) {
             $component = new $class();
@@ -132,8 +133,7 @@ class Horde_iCalendar {
         }
         $found = false;
         if (!$append) {
-            $keys = array_keys($this->_attributes);
-            foreach ($keys as $key) {
+            foreach (array_keys($this->_attributes) as $key) {
                 if ($this->_attributes[$key]['name'] == String::upper($name)) {
                     $this->_attributes[$key]['params'] = $params;
                     $this->_attributes[$key]['value'] = $value;
@@ -219,7 +219,7 @@ class Horde_iCalendar {
      *  b) (unsecapd) comma seperated lists.
      *
      * So for a vcard like "KEY:a,b\nKEY:c" getAttributesValues('KEY')
-     * will return array('a','b','c').
+     * will return array('a', 'b', 'c').
      *
      * @param string  $name    The name of the attribute.
      * @return mixed (object)  PEAR_Error if the attribute does not exist.
@@ -448,14 +448,11 @@ class Horde_iCalendar {
      */
     function isOldFormat()
     {
-        if ($this->_container !== false) {
-            return $this->_container->isOldFormat();
-        }
         if ($this->getType() == 'vcard') {
-            return $this->_version < 3 ? true : false;
+            return ($this->_version < 3);
         }
         if ($this->getType() == 'vNote') {
-            return $this->_version < 2 ? true : false;
+            return ($this->_version < 2);
         }
         if ($this->_version >= 2) {
             return false;
@@ -513,31 +510,36 @@ class Horde_iCalendar {
     /**
      * Parses a string containing vCalendar data.
      *
+     * @todo This method doesn't work well at all, if $base is VCARD.
+     *
      * @param string $text     The data to parse.
      * @param string $base     The type of the base object.
      * @param string $charset  The encoding charset for $text. Defaults to
-     *                         utf-8.
+     *                         utf-8 for new format, iso-8859-1 for old format.
      * @param boolean $clear   If true clears the iCal object before parsing.
      *
      * @return boolean  True on successful import, false otherwise.
      */
-    function parsevCalendar($text, $base = 'VCALENDAR', $charset = 'utf-8',
+    function parsevCalendar($text, $base = 'VCALENDAR', $charset = null,
                             $clear = true)
     {
         if ($clear) {
             $this->clear();
         }
-        if (preg_match('/(BEGIN:' . $base . '\r?\n?)([\W\w]*)(END:' . $base . '\r?\n?)/i', $text, $matches)) {
-            $vCal = $matches[2];
+        if (preg_match('/^BEGIN:' . $base . '(.*)^END:' . $base . '/ism', $text, $matches)) {
+            $container = true;
+            $vCal = $matches[1];
         } else {
             // Text isn't enclosed in BEGIN:VCALENDAR
             // .. END:VCALENDAR. We'll try to parse it anyway.
+            $container = false;
             $vCal = $text;
         }
+        $vCal = trim($vCal);
 
         // All subcomponents.
         $matches = null;
-        if (preg_match_all('/BEGIN:([\W\w]*)(\r\n|\r|\n)([\W\w]*)END:\1(\r\n|\r|\n)/Ui', $vCal, $matches)) {
+        if (preg_match_all('/^BEGIN:(.*)(\r\n|\r|\n)(.*)^END:\1/Uims', $vCal, $matches)) {
             // vTimezone components are processed first. They are
             // needed to process vEvents that may use a TZID.
             foreach ($matches[0] as $key => $data) {
@@ -549,7 +551,7 @@ class Horde_iCalendar {
                 if ($component === false) {
                     return PEAR::raiseError("Unable to create object for type $type");
                 }
-                $component->parsevCalendar($data);
+                $component->parsevCalendar($data, $type, $charset);
 
                 $this->addComponent($component);
 
@@ -567,33 +569,35 @@ class Horde_iCalendar {
                 if ($component === false) {
                     return PEAR::raiseError("Unable to create object for type $type");
                 }
-                $component->parsevCalendar($data);
+                $component->parsevCalendar($data, $type, $charset);
 
                 $this->addComponent($component);
 
                 // Remove from the vCalendar data.
                 $vCal = str_replace($data, '', $vCal);
             }
+        } elseif (!$container) {
+            return false;
         }
 
-        // Unfold any folded lines.
-        $vCal = preg_replace('/[\r\n]+[ \t]/', '', $vCal);
-
         // Unfold "quoted printable" folded lines like:
         //  BODY;ENCODING=QUOTED-PRINTABLE:=
         //  another=20line=
         //  last=20line
-        while (preg_match_all('/^([^:]+;\s*ENCODING=QUOTED-PRINTABLE(.*=\r?\n)+(.*[^=])?\r?\n)/mU', $vCal, $matches)) {
+        while (preg_match_all('/^([^:]+;\s*(ENCODING=)?QUOTED-PRINTABLE(.*=\r?\n)+(.*[^=])?\r?\n)/mU', $vCal, $matches)) {
             foreach ($matches[1] as $s) {
                 $r = preg_replace('/=\r?\n/', '', $s);
                 $vCal = str_replace($s, $r, $vCal);
             }
         }
 
+        // Unfold any folded lines.
+        $vCal = preg_replace('/[\r\n]+[ \t]/', '', $vCal);
+
         // Parse the remaining attributes.
-        if (preg_match_all('/(.*):([^\r\n]*)[\r\n]+/', $vCal, $matches)) {
+        if (preg_match_all('/^((?:[^":]+|(?:"[^"]*")+)*):([^\r\n]*)\r?$/m', $vCal, $matches)) {
             foreach ($matches[0] as $attribute) {
-                preg_match('/([^;^:]*)((;[^:]*)?):([^\r\n]*)[\r\n]*/', $attribute, $parts);
+                preg_match('/([^;^:]*)((;(?:[^":]+|(?:"[^"]*")+)*)?):([^\r\n]*)[\r\n]*/', $attribute, $parts);
                 $tag = String::upper($parts[1]);
                 $value = $parts[4];
                 $params = array();
@@ -608,23 +612,22 @@ class Horde_iCalendar {
                 }
 
                 // Charset and encoding handling.
-                if ((isset($params['ENCODING'])
-                     && String::upper($params['ENCODING']) == 'QUOTED-PRINTABLE')
-                    || isset($params['QUOTED-PRINTABLE'])) {
+                if ((isset($params['ENCODING']) &&
+                     String::upper($params['ENCODING']) == 'QUOTED-PRINTABLE') ||
+                    isset($params['QUOTED-PRINTABLE'])) {
 
                     $value = quoted_printable_decode($value);
-                    // Quoted printable is normally encoded as utf-8.
                     if (isset($params['CHARSET'])) {
                         $value = String::convertCharset($value, $params['CHARSET']);
                     } else {
-                        $value = String::convertCharset($value, 'utf-8');
+                        $value = String::convertCharset($value, empty($charset) ? ($this->isOldFormat() ? 'iso-8859-1' : 'utf-8') : $charset);
                     }
                 } elseif (isset($params['CHARSET'])) {
                     $value = String::convertCharset($value, $params['CHARSET']);
                 } else {
                     // As per RFC 2279, assume UTF8 if we don't have an
                     // explicit charset parameter.
-                    $value = String::convertCharset($value, $charset);
+                    $value = String::convertCharset($value, empty($charset) ? ($this->isOldFormat() ? 'iso-8859-1' : 'utf-8') : $charset);
                 }
 
                 // Get timezone info for date fields from $params.
@@ -639,6 +642,7 @@ class Horde_iCalendar {
                     break;
 
                 case 'BDAY':
+                case 'X-SYNCJE-ANNIVERSARY':
                     $this->setAttribute($tag, $this->_parseDate($value), $params);
                     break;
 
@@ -648,10 +652,13 @@ class Horde_iCalendar {
                 case 'DUE':
                 case 'AALARM':
                 case 'RECURRENCE-ID':
+                    // types like AALARM may contain additional data after a ;
+                    // ignore these.
+                    $ts = explode(';', $value);
                     if (isset($params['VALUE']) && $params['VALUE'] == 'DATE') {
-                        $this->setAttribute($tag, $this->_parseDate($value), $params);
+                        $this->setAttribute($tag, $this->_parseDate($ts[0]), $params);
                     } else {
-                        $this->setAttribute($tag, $this->_parseDateTime($value, $tzid), $params);
+                        $this->setAttribute($tag, $this->_parseDateTime($ts[0], $tzid), $params);
                     }
                     break;
 
@@ -711,9 +718,15 @@ class Horde_iCalendar {
 
                 // Geo fields.
                 case 'GEO':
-                    $floats = explode(';', $value);
-                    $value['latitude'] = floatval($floats[0]);
-                    $value['longitude'] = floatval($floats[1]);
+                    if ($this->isOldFormat()) {
+                        $floats = explode(',', $value);
+                        $value = array('latitude' => floatval($floats[1]),
+                                       'longitude' => floatval($floats[0]));
+                    } else {
+                        $floats = explode(';', $value);
+                        $value = array('latitude' => floatval($floats[0]),
+                                       'longitude' => floatval($floats[1]));
+                    }
                     $this->setAttribute($tag, $value, $params);
                     break;
 
@@ -745,9 +758,8 @@ class Horde_iCalendar {
                 // String fields.
                 default:
                     if ($this->isOldFormat()) {
-                        // vCalendar 1.0 and vcCard 2.1 only escape
-                        // semicolons and use unescaped semicolons to
-                        // create lists.
+                        // vCalendar 1.0 and vCard 2.1 only escape semicolons
+                        // and use unescaped semicolons to create lists.
                         $value = trim($value);
                         // Split by unescaped semicolons:
                         $values = preg_split('/(?<!\\\\);/', $value);
@@ -756,14 +768,14 @@ class Horde_iCalendar {
                         $this->setAttribute($tag, trim($value), $params, true, $values);
                     } else {
                         $value = trim($value);
-                        // As of rfc 2426 2.4.2 semicolon, comma, and
-                        // colon must be escaped (comma is unescaped after
-                        // splitting below).
+                        // As of rfc 2426 2.4.2 semicolon, comma, and colon
+                        // must be escaped (comma is unescaped after splitting
+                        // below).
                         $value = str_replace(array('\\n', '\\N', '\\;', '\\:', '\\\\'),
                                              array($this->_newline, $this->_newline, ';', ':', '\\'),
                                              $value);
 
-                        // Split by unescaped commas:
+                        // Split by unescaped commas.
                         $values = preg_split('/(?<!\\\\),/', $value);
                         $value = str_replace('\\,', ',', $value);
                         $values = str_replace('\\,', ',', $values);
@@ -813,7 +825,7 @@ class Horde_iCalendar {
                     }
                     /* Skip VALUE=DATE for vCalendar 1.0 data, not allowed. */
                     if ($this->isOldFormat() &&
-                         $param_name == 'VALUE' && $param_value == 'DATE') {
+                        $param_name == 'VALUE' && $param_value == 'DATE') {
                         continue;
                     }
 
@@ -843,7 +855,18 @@ class Horde_iCalendar {
             case 'RECURRENCE-ID':
                 if (isset($params['VALUE'])) {
                     if ($params['VALUE'] == 'DATE') {
-                        $value = $this->_exportDate($value, $name == 'DTEND' ? '235959' : '000000');
+                        // VCALENDAR 1.0 uses T000000 - T235959 for all day events:
+                        if ($this->isOldFormat() && $name == 'DTEND') {
+                            $d = new Horde_Date($value);
+                            $value = new Horde_Date(array(
+                                'year' => $d->year,
+                                'month' => $d->month,
+                                'mday' => $d->mday - 1));
+                            $value->correct();
+                            $value = $this->_exportDate($value, '235959');
+                        } else {
+                            $value = $this->_exportDate($value, '000000');
+                        }
                     } else {
                         $value = $this->_exportDateTime($value);
                     }
@@ -915,7 +938,11 @@ class Horde_iCalendar {
 
             // Geo fields.
             case 'GEO':
-                $value = $value['latitude'] . ',' . $value['longitude'];
+                if ($this->isOldFormat()) {
+                    $value = $value['longitude'] . ',' . $value['latitude'];
+                } else {
+                    $value = $value['latitude'] . ';' . $value['longitude'];
+                }
                 break;
 
             // Recurrence fields.
@@ -943,8 +970,7 @@ class Horde_iCalendar {
                     // Text containing newlines or ASCII >= 127 must be BASE64
                     // or QUOTED-PRINTABLE encoded. Currently we use
                     // QUOTED-PRINTABLE as default.
-                    // FIXME: deal with base64 encodings!
-                    if (preg_match("/[^\x20-\x7F\r\n]/", $value) &&
+                    if (preg_match("/[^\x20-\x7F]/", $value) &&
                         empty($params['ENCODING']))  {
                         $params['ENCODING'] = 'QUOTED-PRINTABLE';
                         $params_str .= ';ENCODING=QUOTED-PRINTABLE';
@@ -957,7 +983,7 @@ class Horde_iCalendar {
                     }
                 } else {
                     if (is_array($attribute['values']) &&
-                        count($attribute['values']) > 1) {
+                        count($attribute['values'])) {
                         $values = $attribute['values'];
                         if ($name == 'N' || $name == 'ADR' || $name == 'ORG') {
                             $glue = ';';
@@ -966,34 +992,37 @@ class Horde_iCalendar {
                         }
                         // As of rfc 2426 2.5 semicolon and comma must be
                         // escaped.
-                        $values = str_replace(array(';', ',', '\\'),
-                                              array('\\;', '\\,', '\\\\'),
+                        $values = str_replace(array('\\', ';', ','),
+                                              array('\\\\', '\\;', '\\,'),
                                               $values);
                         $value = implode($glue, $values);
                     } else {
                         // As of rfc 2426 2.5 semicolon and comma must be
                         // escaped.
-                        $value = str_replace(array(';', ',', '\\'),
-                                             array('\\;', '\\,', '\\\\'),
+                        $value = str_replace(array('\\', ';', ','),
+                                             array('\\\\', '\\;', '\\,'),
                                              $value);
                     }
+                    $value = preg_replace('/\r?\n/', '\n', $value);
                 }
                 break;
             }
 
+            $value = str_replace("\r", '', $value);
             if (!empty($params['ENCODING']) &&
                 $params['ENCODING'] == 'QUOTED-PRINTABLE' &&
                 strlen(trim($value))) {
-                /* quotedPrintableEncode does not escape CRLFs, but strange
-                 * enough single LFs. so convert everything to LF only and
-                 * replace afterwards. */
-                $value = str_replace("\r", '', $value);
-                $result .= "$name$params_str:=" . $this->_newline
-                    . str_replace('=0A', '=0D=0A', $this->_quotedPrintableEncode($value))
+                $result .= $name . $params_str . ':'
+                    . str_replace('=0A', '=0D=0A',
+                                  $this->_quotedPrintableEncode($value))
                     . $this->_newline;
             } else {
-                $attr_string = "$name$params_str:$value";
-                $result .= $this->_foldLine($attr_string) . $this->_newline;
+                $attr_string = $name . $params_str . ':' . $value;
+                if (!$this->isOldFormat()) {
+                    $attr_string = String::wordwrap($attr_string, 75, $this->_newline . ' ',
+                                                    true, 'utf-8', true);
+                }
+                $result .= $attr_string . $this->_newline;
             }
         }
 
@@ -1144,8 +1173,8 @@ class Horde_iCalendar {
         }
 
         // Get timezone info for date fields from $tzid and container.
-        $tzoffset = ($time['zone'] == 'Local' && $tzid && is_a($this->_container, 'Horde_iCalendar')) ?
-            $this->_parseTZID($date, $time, $tzid) : false;
+        $tzoffset = ($time['zone'] == 'Local' && $tzid && is_a($this->_container, 'Horde_iCalendar'))
+            ? $this->_parseTZID($date, $time, $tzid) : false;
         if ($time['zone'] == 'UTC' || $tzoffset !== false) {
             $result = @gmmktime($time['hour'], $time['minute'], $time['second'],
                                 $date['month'], $date['mday'], $date['year']);
@@ -1352,33 +1381,10 @@ class Horde_iCalendar {
     }
 
     /**
-     * Return the folded version of a line.
-     */
-    function _foldLine($line)
-    {
-        $line = preg_replace("/\r\n|\n|\r/", '\n', $line);
-        if (strlen($line) > 75) {
-            $foldedline = '';
-            while (!empty($line)) {
-                $maxLine = substr($line, 0, 75);
-                $cutPoint = max(60, max(strrpos($maxLine, ';'), strrpos($maxLine, ':')) + 1);
-
-                $foldedline .= (empty($foldedline)) ?
-                    substr($line, 0, $cutPoint) :
-                    $this->_newline . ' ' . substr($line, 0, $cutPoint);
-
-                $line = (strlen($line) <= $cutPoint) ? '' : substr($line, $cutPoint);
-            }
-            return $foldedline;
-        }
-        return $line;
-    }
-
-    /**
-     * Convert an 8bit string to a quoted-printable string according
-     * to RFC2045, section 6.7.
+     * Converts an 8bit string to a quoted-printable string according to RFC
+     * 2045, section 6.7.
      *
-     * Uses imap_8bit if available.
+     * imap_8bit() does not apply all necessary rules.
      *
      * @param string $input  The string to be encoded.
      *
@@ -1386,25 +1392,49 @@ class Horde_iCalendar {
      */
     function _quotedPrintableEncode($input = '')
     {
-        // If imap_8bit() is available, use it.
-        if (function_exists('imap_8bit')) {
-            return imap_8bit($input);
-        }
-
-        // Rather dumb replacment: just encode everything.
-        $hex = array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
-                     'A', 'B', 'C', 'D', 'E', 'F');
-
-        $output = '';
+        $output = $line = '';
         $len = strlen($input);
+
         for ($i = 0; $i < $len; ++$i) {
-            $c = substr($input, $i, 1);
-            $dec = ord($c);
-            $output .= '=' . $hex[floor($dec / 16)] . $hex[floor($dec % 16)];
-            if (($i + 1) % 25 == 0) {
-                $output .= "=\r\n";
+            $ord = ord($input[$i]);
+            // Encode non-printable characters (rule 2).
+            if ($ord == 9 ||
+                ($ord >= 32 && $ord <= 60) ||
+                ($ord >= 62 && $ord <= 126)) {
+                $chunk = $input[$i];
+            } else {
+                // Quoted printable encoding (rule 1).
+                $chunk = '=' . String::upper(sprintf('%02X', $ord));
+            }
+            $line .= $chunk;
+            // Wrap long lines (rule 5)
+            if (strlen($line) + 1 > 76) {
+                $line = String::wordwrap($line, 75, "=\r\n", true, 'us-ascii', true);
+                $newline = strrchr($line, "\r\n");
+                if ($newline !== false) {
+                    $output .= substr($line, 0, -strlen($newline) + 2);
+                    $line = substr($newline, 2);
+                } else {
+                    $output .= $line;
+                }
+                continue;
+            }
+            // Wrap at line breaks for better readability (rule 4).
+            if (substr($line, -3) == '=0A') {
+                $output .= $line . "=\r\n";
+                $line = '';
             }
         }
+        $output .= $line;
+
+        // Trailing whitespace must be encoded (rule 3).
+        $lastpos = strlen($output) - 1;
+        if ($output[$lastpos] == chr(9) ||
+            $output[$lastpos] == chr(32)) {
+            $output[$lastpos] = '=';
+            $output .= String::upper(sprintf('%02X', ord($output[$lastpos])));
+        }
+
         return $output;
     }
 
