@@ -2,9 +2,9 @@
 /**
  * Class representing vJournals.
  *
- * $Horde: framework/iCalendar/iCalendar/vjournal.php,v 1.8.10.2 2005/01/16 23:10:41 chuck Exp $
+ * $Horde: framework/iCalendar/iCalendar/vjournal.php,v 1.8.10.8 2008/07/03 08:42:58 jan Exp $
  *
- * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -20,11 +20,6 @@ class Horde_iCalendar_vjournal extends Horde_iCalendar {
         return 'vJournal';
     }
 
-    function parsevCalendar($data)
-    {
-        parent::parsevCalendar($data, 'VJOURNAL');
-    }
-
     function exportvCalendar()
     {
         return parent::_exportvData('VJOURNAL');
