@@ -13,9 +13,9 @@ require_once 'Net/IMSP/Auth.php';
  *   'server'       The hostname of the IMSP server.
  *   'port'         The port of the IMSP server.</pre>
  *
- * $Horde: framework/Net_IMSP/IMSP/Options.php,v 1.3.2.10 2005/09/29 00:50:38 selsky Exp $
+ * $Horde: framework/Net_IMSP/IMSP/Options.php,v 1.3.2.19 2008/01/02 11:30:45 jan Exp $
  *
- * Copyright 2004-2005 Michael Rubinsky <mrubinsk@horde.org>
+ * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -65,6 +65,7 @@ class Net_IMSP_Options {
         if (is_a($this->_imsp, 'PEAR_Error')) {
             return $this->_imsp;
         }
+
         $this->_imsp->writeToLog('Net_IMSP_Options initialized.', __FILE__,
                                  __LINE__, PEAR_LOG_DEBUG);
         return true;
@@ -75,6 +76,7 @@ class Net_IMSP_Options {
      *
      * @param  string $optionName Name of option to retrieve. Accepts '*'
      *                            as wild card.
+     *
      * @return mixed  Associative array containing option=>value pairs or
      *                PEAR_Error.
      */
@@ -136,7 +138,7 @@ class Net_IMSP_Options {
         }
 
         if ($server_response != 'OK') {
-            return $this->_imsp->imspError(IMSP_UNEXPECTED_RESPONSE,
+            return $this->_imsp->imspError('Did not receive the expected response from the server.',
                                            __FILE__, __LINE__);
         }
 
@@ -147,8 +149,9 @@ class Net_IMSP_Options {
     /**
      * Function sets an option value on the IMSP server.
      *
-     * @param string $optionName Name of option to set.
+     * @param string $optionName  Name of option to set.
      * @param string $optionValue Value to assign.
+     *
      * @return mixed True or PEAR_Error.
      */
     function set($optionName, $optionValue)
@@ -164,9 +167,9 @@ class Net_IMSP_Options {
                 return $result;
             }
 
-            if (!preg_match(IMSP_COMMAND_CONTINUATION_RESPONSE,
+            if (!preg_match("/^\+/",
                             $this->_imsp->imspReceive())) {
-                return $this->_imsp->imspError(IMSP_NO_CONTINUATION_RESPONSE,
+                return $this->_imsp->imspError('Did not receive expected command continuation response from IMSP server.',
                                                __FILE__, __LINE__);
             }
         }
@@ -204,10 +207,10 @@ class Net_IMSP_Options {
             return $this->_imsp->imspError('The IMSP log could not be initialized.');
         }
     }
+
     function logout()
     {
         $this->_imsp->logout();
     }
 
-
 }
