@@ -5,24 +5,22 @@ require_once dirname(__FILE__) . '/../VFS.php';
 /**
  * A wrapper for the VFS class to return objects, instead of arrays.
  *
- * $Horde: framework/VFS/VFS/Object.php,v 1.12 2004/04/08 18:33:17 slusarz Exp $
+ * $Horde: framework/VFS/lib/VFS/Object.php,v 1.1.2.1 2007/12/20 13:50:21 jan Exp $
  *
- * Copyright 2002-2004 Jon Wood <jon@jellybob.co.uk>
+ * Copyright 2002-2007 Jon Wood <jon@jellybob.co.uk>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Jon Wood <jon@jellybob.co.uk>
- * @version $Revision: 1.12 $
  * @package VFS
- * @since   Horde 2.2
  */
 class VFS_Object {
 
     /**
      * The actual vfs that does the work
      *
-     * @var object VFS $_vfs
+     * @var VFS
      */
     var $_vfs;
 
@@ -30,7 +28,7 @@ class VFS_Object {
      * The current path that has been passed to listFolder, if this
      * changes, the list will be rebuilt.
      *
-     * @var string $_currentPath
+     * @var string
      */
     var $_currentPath;
 
@@ -38,7 +36,7 @@ class VFS_Object {
      * The return value from a standard VFS listFolder call, to
      * be read with the Object listFolder.
      *
-     * @var array $_folderList
+     * @var array
      */
     var $_folderList;
 
@@ -48,9 +46,7 @@ class VFS_Object {
      * If you pass in an existing VFS object, it will be used as the VFS
      * object for this object.
      *
-     * @access public
-     *
-     * @param object VFS &$vfs  The VFS object to wrap.
+     * @param VFS &$vfs  The VFS object to wrap.
      */
     function VFS_Object(&$vfs)
     {
@@ -60,29 +56,23 @@ class VFS_Object {
     }
 
     /**
-     * Attempts to return a concrete VFS_Object instance based on
-     * $driver.
+     * Attempts to return a concrete VFS_Object instance based on $driver.
      *
-     * @access public
+     * @param mixed $driver  The type of concrete VFS subclass to return. If
+     *                       $driver is an array then we will look in
+     *                       $driver[0]/lib/VFS/ for the subclass
+     *                       implementation named $driver[1].php.
+     * @param array $params  A hash containing any additional configuration or
+     *                       connection parameters a subclass might need.
      *
-     * @param mixed $driver           The type of concrete VFS subclass to
-     *                                return. This is based on the storage
-     *                                driver ($driver). The code is
-     *                                dynamically included. If $driver is an
-     *                                array then we will look in
-     *                                $driver[0]/lib/VFS/ for the subclass
-     *                                implementation named $driver[1].php.
-     * @param optional array $params  A hash containing any additional
-     *                                configuration or connection parameters
-     *                                a subclass might need.
-     *
-     * @return object VFS_Object  The newly created concrete VFS_Object
-     *                            instance, or false on an error.
+     * @return VFS_Object  The newly created concrete VFS_Object instance, or
+     *                     false on an error.
      */
     function &factory($driver, $params = array())
     {
         $vfs = &VFS::factory($driver, $params = array());
-        return $ret = &new VFS_Object($vfs);
+        $vfsobject = &new VFS_Object($vfs);
+        return $vfsobject;
     }
 
     /**
@@ -95,33 +85,26 @@ class VFS_Object {
      *
      * This method must be invoked as: $var = &VFS::singleton()
      *
-     * @access public
+     * @param mixed $driver  The type of concrete VFS subclass to return. If
+     *                       $driver is an array then we will look in
+     *                       $driver[0]/lib/VFS/ for the subclass
+     *                       implementation named $driver[1].php.
+     * @param array $params  A hash containing any additional configuration or
+     *                       connection parameters a subclass might need.
      *
-     * @param mixed $driver           The type of concrete VFS subclass to
-     *                                return. This is based on the storage
-     *                                driver ($driver). The code is
-     *                                dynamically included. If $driver is an
-     *                                array then we will look in
-     *                                $driver[0]/lib/VFS/ for the subclass
-     *                                implementation named $driver[1].php.
-     * @param optional array $params  A hash containing any additional
-     *                                configuration or connection parameters
-     *                                a subclass might need.
-     *
-     * @return object VFS_Object  The concrete VFS_Object reference, or false
-     *                            on error.
+     * @return VFS_Object  The concrete VFS_Object reference, or false on
+     *                     error.
      */
     function &singleton($driver, $params = array())
     {
         $vfs = &VFS::singleton($driver, $params = array());
-        return $ret = &new VFS_Object($vfs);
+        $vfsobject = &new VFS_Object($vfs);
+        return $vfsobject;
     }
 
     /**
      * Check the credentials that we have to see if there is a valid login.
      *
-     * @access public
-     *
      * @return mixed  True on success, PEAR_Error describing the problem
      *                if the credentials are invalid.
      */
@@ -133,10 +116,8 @@ class VFS_Object {
     /**
      * Set configuration parameters.
      *
-     * @access public
-     *
-     * @param optional array $params  An associative array:
-     *                                KEY: param name, VAL: param value
+     * @param array $params  An associative array of parameter name/value
+     *                       pairs.
      */
     function setParams($params = array())
     {
@@ -146,8 +127,6 @@ class VFS_Object {
     /**
      * Retrieve a file from the VFS.
      *
-     * @access public
-     *
      * @param string $path  The pathname to the file.
      *
      * @return string  The file data.
@@ -160,12 +139,10 @@ class VFS_Object {
     /**
      * Store a file in the VFS.
      *
-     * @access public
-     *
-     * @param string $path                  The path to store the file in.
-     * @param string $tmpFile               The temporary file containing the 
-     *                                      data to be stored.
-     * @param optional boolean $autocreate  Automatically create directories?
+     * @param string $path         The path to store the file in.
+     * @param string $tmpFile      The temporary file containing the data to be
+     *                             stored.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -177,11 +154,9 @@ class VFS_Object {
     /**
      * Store a file in the VFS from raw data.
      *
-     * @access public
-     *
-     * @param string $path                  The path to store the file in.
-     * @param string $data                  The file data.
-     * @param optional boolean $autocreate  Automatically create directories?
+     * @param string $path         The path to store the file in.
+     * @param string $data         The file data.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
@@ -221,8 +196,6 @@ class VFS_Object {
     /**
      * Create a folder in the VFS.
      *
-     * @access public
-     *
      * @param string $path  The path to the folder.
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
@@ -235,8 +208,6 @@ class VFS_Object {
     /**
      * Deletes a folder from the VFS.
      *
-     * @access public
-     *
      * @param string $path The path of the folder to delete.
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
@@ -251,8 +222,6 @@ class VFS_Object {
      * be read, or a PEAR_Error if it can't be. Returns false once
      * the folder has been completely read.
      *
-     * @access public
-     *
      * @param string $path  The path of the diretory.
      *
      * @return mixed  File list (array) on success, a PEAR_Error
@@ -283,8 +252,6 @@ class VFS_Object {
     /**
      * Changes permissions for an Item on the VFS.
      *
-     * @access public
-     *
      * @param string $path        Holds the path of directory of the Item.
      * @param string $permission  TODO
      *
@@ -298,8 +265,6 @@ class VFS_Object {
     /**
      * Return the list of additional credentials required, if any.
      *
-     * @access public
-     *
      * @return array  Credential list.
      */
     function getRequiredCredentials()
@@ -308,10 +273,8 @@ class VFS_Object {
     }
 
     /**
-     * Return the array specificying what permissions are
-     * changeable for this implementation.
-     *
-     * @access public
+     * Return the array specificying what permissions are changeable for this
+     * implementation.
      *
      * @return array  Changeable permisions.
      */
@@ -320,14 +283,4 @@ class VFS_Object {
         return $this->_vfs->getModifiablePermissions();
     }
 
-    /**
-     * Close any resources that need to be closed.
-     *
-     * @access private
-     */
-    function _disconnect()
-    {
-        $this->_vfs->_disconnect();
-    }
-
 }
