@@ -1,11 +1,10 @@
 <?php
 /**
- * $Horde: horde/lib/prefs.php,v 1.19.4.13 2007/02/09 14:05:40 jan Exp $
+ * $Horde: horde/lib/prefs.php,v 1.19.4.18 2008/08/22 11:23:38 jan Exp $
  *
- * Copyright 1999-2007 Charles J. Hagenbuch <chuck@horde.org>
- * Copyright 1999-2007 Jon Parise <jon@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
+ * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  */
 
@@ -38,20 +37,20 @@ function handle_themeselect($updated)
 function handle_categorymanagement($updated)
 {
     require_once 'Horde/Prefs/CategoryManager.php';
-    $cManager = &new Prefs_CategoryManager();
+    $cManager = new Prefs_CategoryManager();
 
     /* Always save colors of all categories. */
     $colors = array();
     $categories = $cManager->get();
     foreach ($categories as $category) {
-        if ($color = Util::getFormData('color_' . base64_encode($category))) {
+        if ($color = Util::getFormData('color_' . md5($category))) {
             $colors[$category] = $color;
         }
     }
-    if ($color = Util::getFormData('color_' . base64_encode('_default_'))) {
+    if ($color = Util::getFormData('color_' . md5('_default_'))) {
         $colors['_default_'] = $color;
     }
-    if ($color = Util::getFormData('color_' . base64_encode('_unfiled_'))) {
+    if ($color = Util::getFormData('color_' . md5('_unfiled_'))) {
         $colors['_unfiled_'] = $color;
     }
     $cManager->setColors($colors);
@@ -76,6 +75,19 @@ function handle_categorymanagement($updated)
     return $updated;
 }
 
+function handle_credentialsui($updated)
+{
+    global $prefs;
+
+    $credentials = Util::getFormData('credentials');
+    if (!is_null($credentials)) {
+        $prefs->setValue('credentials', serialize($credentials));
+        $updated = true;
+    }
+
+    return $updated;
+}
+
 /**
  * Do anything that we need to do as a result of certain preferences
  * changing.
@@ -87,9 +99,12 @@ function prefs_callback()
     if ($prefs->isDirty('language') ||
         $prefs->isDirty('show_sidebar')) {
         if ($prefs->isDirty('language')) {
-            NLS::setLang($prefs->getValue('language'));
-            NLS::setTextdomain($registry->getApp(), $registry->get('fileroot') . '/locale', NLS::getCharset());
-            String::setDefaultCharset(NLS::getCharset());
+            NLS::setLanguageEnvironment($prefs->getValue('language'));
+            foreach ($registry->listAPIs() as $api) {
+                if ($registry->hasMethod($api . '/changeLanguage')) {
+                    $registry->call($api . '/changeLanguage');
+                }
+            }
         }
 
         $url = $registry->get('webroot', 'horde');
@@ -106,7 +121,7 @@ function prefs_callback()
     }
 
     if ($prefs->isDirty('sidebar_width')) {
-        $notification->push('if (window.parent && window.parent.document.getElementById(\'hf\') && window.parent.horde_menu && window.parent.horde_menu.expandedSidebar.shown()) window.parent.document.getElementById(\'hf\').cols = window.parent.horde_menu.rtl ? \'*,' . $prefs->getValue('sidebar_width') . '\' : \'' . $prefs->getValue('sidebar_width') . ',*\';', 'javascript');
+        $notification->push('if (window.parent && window.parent.document.getElementById(\'hf\') && window.parent.horde_menu && window.parent.horde_menu.document.getElementById(\'expandedSidebar\').style.display != \'hidden\') window.parent.document.getElementById(\'hf\').cols = window.parent.horde_menu.rtl ? \'*,' . $prefs->getValue('sidebar_width') . '\' : \'' . $prefs->getValue('sidebar_width') . ',*\';', 'javascript');
     }
 
     if ($prefs->isDirty('theme') ||
@@ -148,7 +163,9 @@ if (!$prefs->isLocked('theme')) {
             }
 
             $theme_name = null;
-            @include $theme_base . '/' . $dir . '/info.php';
+            if (is_readable($theme_base . '/' . $dir . '/info.php')) {
+                include $theme_base . '/' . $dir . '/info.php';
+            }
             if (!empty($theme_name)) {
                 $theme_options[$dir] = $theme_name;
             }
