@@ -1,11 +1,13 @@
 <?php
 /**
- * $Horde: horde/services/portal/rpcsum.php,v 2.26.6.1 2005/01/03 12:25:46 jan Exp $
+ * $Horde: horde/services/portal/rpcsum.php,v 2.26.6.7 2008/04/07 20:58:08 slusarz Exp $
  *
- * Copyright 2001-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2001-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL).  If you
+ * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author Jan Schneider <jan@horde.org>
  */
 
 @define('HORDE_BASE', dirname(__FILE__) . '/../..');
@@ -84,9 +86,15 @@ case 'delete':
 
 /* Show the header. */
 require_once 'Horde/Prefs/UI.php';
-require HORDE_BASE . '/config/prefs.php';
+$result = Horde::loadConfiguration('prefs.php', array('prefGroups', '_prefs'), 'horde');
+if (!is_a($result, 'PEAR_Error')) {
+    extract($result);
+}
 $app = 'horde';
-Prefs_UI::generateHeader('remote');
+$chunk = Util::nonInputVar('chunk');
+Prefs_UI::generateHeader('remote', $chunk);
 
 require HORDE_TEMPLATES . '/rpcsum/rpcsum.inc';
-require HORDE_TEMPLATES . '/common-footer.inc';
+if (!$chunk) {
+    require HORDE_TEMPLATES . '/common-footer.inc';
+}
